cd $TOP
file=$objdir/Makefile

echo Extracting $file
if test ! -d $objdir; then mkdir $objdir; fi
rm -f $file

# Private directories:
src=../src

list="\
  kernel\
  mpfile\
  alglin1\
  alglin2\
  anal\
  arith1\
  arith2\
  base1\
  base2\
  base3\
  base4\
  base5\
  bibli1\
  bibli2\
  buch1\
  buch2\
  buch3\
  buch4\
  compat\
  elliptic\
  errmessages\
  es\
  gen1\
  gen2\
  gen3\
  gp\
  gp_init\
  gp_rl\
  helpmessages\
  init\
  nffactor\
  plot\
  plotport\
  polarit1\
  polarit2\
  rootpol\
  subfields\
  thue\
  trans1\
  trans2\
  trans3\
"

case "$arch" in
  none) head="# General Makefile for C version of Pari/GP";;
  m68k) head="# Makefile for Pari/GP on MC680x0 (x>=2) running $osname";;
     *) head="# Makefile for Pari/GP on $arch running $osname";;
esac

#KERNEL            # Kernel of PARI (asm routines)
#ASMFILE           # Kernel Inline, for sparc/gcc only, defined in config.h
MPFILE="mp.c"      # Multi-precision for PARI (asm if m68k)

if [ "$osname" = solaris ]; then as="$as -T"; fi
if [ "$arch"   = sparcv9 ]; then as="$as -xarch=v8plus"; fi

create_asmfile=''; depend_asmfile=''
case "$asmarch" in
  i386)   # KERNEL is not in src/asmfiles but in $objdir, because built by make.
          KERNEL="$asmarch.s";                 kbuild="\$(CC) \$(CFLAGS) -c";
          depend_asmfile="$src/asmfiles/i386.c"
          create_asmfile="\$(CC) -E $src/asmfiles/i386.c > $KERNEL";;
  alpha)  KERNEL="$src/asmfiles/$asmarch.s";   kbuild="$as -01";;
  hppa)   KERNEL="$src/asmfiles/$asmarch.s";   kbuild="$cc -c";;
  sparc*) KERNEL="$src/asmfiles/$asmarch.s";   kbuild="$as -P -I.";;
  m68k)   MPFILE="asmfiles/mp.s"
          KERNEL="$src/version/version68k.c";  kbuild="\$(CC) \$(CFLAGS) -c";;
  none)   KERNEL="$src/version/versionport.c"; kbuild="\$(CC) \$(CFLAGS) -c";;
  *)      KERNEL="$src/version/versionport.c"; kbuild="\$(CC) \$(CFLAGS) -c"
          echo "        Warning ! no asm file for architecture $asmarch"
          asmarch=none;;
esac

OBJS=''; OBJSGP=''; OBJSLIB=''
for f in $list; do
  case $f in
  plot|plotport|gp|gp_rl|gp_init)
    OBJSGP="$OBJSGP $f.o";;
  version)
    case $VERSIONFILE in
      version/version*) OBJS="$OBJS $f.o";;
    esac ;;
  *) OBJS="$OBJS $f.o";;
  esac
done  

case "$arch" in
  hppa)      dynlib=sl ;;
  *)         dynlib=so ;;
esac

case "$which_graphic_lib" in
none)
  PLOTFILE=plot/plotnull.c
  PLOTCFLAGS=
  PLOTLIBS=
  plotrunpath=
  ;;
sunview)
  PLOTFILE=plot/plotsun.c
  PLOTCFLAGS=
  PLOTLIBS='-lsuntool -lsunwindow -lpixrect'
  plotrunpath=
  ;;
X11)
  PLOTFILE=plot/plotX.c
  PLOTCFLAGS=-I$Xincroot
  PLOTLIBS="-L$X11 -lX11 $extralib"
  plotrunpath=$X11
  ;;
esac
## Gnuplot ???

case "$readline" in
  '')rr="#";;
  *) rr='';;
esac

case "$runpathprefix" in
  -R) RUNPTH=
      for dir in $TOP/$objdir $runpath $plotrunpath;
        do RUNPTH="${RUNPTH}-R$dir "; done ;; 
  *-rpath*) RUNPTH=$runpathprefix
      for dir in $TOP/$objdir $runpath $plotrunpath;
        do RUNPTH="${RUNPTH}$dir:"; done ;;
esac

case "$static" in
  n) dft=dyn ;;
  y) dft=sta ;;
esac

cat > $file << EOT
$head
#
#  This file was created by Configure. Any change made to it will be
#  lost when Configure is run.
#

CC         = $CC
CFLAGS     = $CFLAGS -I. -I$src
LD         = $LD
LDFLAGS    = $LDFLAGS
DLLD       = $DLLD
DLLDFLAGS  = $DLLDFLAGS

RUNPTH     = $RUNPTH
LDDYN      = -L$TOP/$objdir $LDDYN
DLCFLAGS   = $DLCFLAGS
LIBS = -lm $LIBS

RM = rm -f
LN = ln -s

# Change these installation directories to suit your needs.

LIBDIR = $libdir
BINDIR = $bindir
MANDIR = $mandir
INCLUDEDIR = $includedir

INSTALL = $config_dir/install
INSTALL_PROGRAM = \$(INSTALL)
INSTALL_DATA = \$(INSTALL) -m 644

# Graphic library.
PLOTFILE   = $PLOTFILE
PLOTCFLAGS = $PLOTCFLAGS
PLOTLIBS   = $PLOTLIBS

# Uncomment to use GNU readline.
$rr RLINCLUDE = $RLINCLUDE
$rr RLLIBS    = $RLLIBS

OBJS   = $OBJS
OBJSGP = $OBJSGP

dft: $dft ../gp

../gp: gp-$dft
	-\$(RM) \$@
	-\$(LN) $objdir/gp-$dft \$@

all: sta dyn

sta: gp-sta libpari.a
dyn: gp-dyn libpari.$dynlib

libpari.a: \$(OBJS)
	-\$(RM) \$@
	ar r \$@  \$(OBJS)
EOT
if test -n "$ranlib"; then
cat >> $file << EOT
	$ranlib \$@
EOT
fi
cat >> $file << EOT

libpari.$dynlib.$version: \$(OBJS)
	\$(DLLD) -o \$@ \$(DLLDFLAGS) \$(OBJS)

libpari.$dynlib: libpari.$dynlib.$version
	-\$(RM) \$@
	-\$(LN) libpari.$dynlib.$version \$@

gp-sta: \$(OBJS) \$(OBJSGP)
	\$(LD) -o \$@ \$(LDFLAGS) \$(OBJS) \$(OBJSGP) \$(RUNPTH) \$(PLOTLIBS) \$(RLLIBS) \$(LIBS)

gp-dyn: \$(OBJSGP) libpari.$dynlib
	\$(LD) -o \$@ \$(LDFLAGS) \$(OBJSGP) \$(RUNPTH) \$(LDDYN) \$(PLOTLIBS) \$(RLLIBS) \$(LIBS)

test: gp-sta gp-dyn dotest

test-compat: gp-sta gp-dyn dotest-compat

cleantest:
	\$(RM) out.* pari.ps

dotest::
	@$src/test/dotest $sizeof_long

dotest-compat::
	@$src/test/dotest $sizeof_long -c

install: install-lib$dft install-include install-bin install-man

install-libdyn: libpari.$dynlib.$version
	-mkdir -p \$(LIBDIR)
	-\$(RM) \$(LIBDIR)/libpari.$dynlib
	\$(INSTALL_DATA) libpari.$dynlib.$version \$(LIBDIR)/libpari.$dynlib.$version
	-\$(LN) libpari.$dynlib.$version \$(LIBDIR)/libpari.$dynlib

install-libsta: libpari.a
	-mkdir -p \$(LIBDIR)
	-\$(RM) \$(LIBDIR)/libpari.a
	\$(INSTALL_DATA) libpari.a \$(LIBDIR)/libpari.a.$version
	-\$(LN) libpari.a.$version \$(LIBDIR)/libpari.a

install-include:
	-mkdir -p \$(INCLUDEDIR)
	\$(INSTALL_DATA) config.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/genpari.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/gencom.h  \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/gen68k.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/errmessages.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/mpansi.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/mpdefs.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/mpinline.h \$(INCLUDEDIR)
	\$(INSTALL_DATA) $src/types.h \$(INCLUDEDIR)

install-bin: gp-$dft
	-mkdir -p \$(BINDIR)
	-\$(RM) \$(BINDIR)/gp
	\$(INSTALL_PROGRAM) gp-$dft \$(BINDIR)/gp-$version
	strip \$(BINDIR)/gp-$version
	-\$(LN) gp-$version \$(BINDIR)/gp

install-man::
	-mkdir -p \$(MANDIR)
	\$(INSTALL_DATA) doc/gp.1 \$(MANDIR)

clean: cleantest
	-\$(RM) *.o

EOT

if test -n "$create_asmfile"; then
cat >> $file << EOT
$KERNEL:	$depend_asmfile
	$create_asmfile
EOT
fi

for f in $list; do
  case "$f" in
  kernel) cat >> $file  <<!!
$f.o:	$KERNEL
	$kbuild -o \$@ $KERNEL
!!
;;
  mpfile) cat >> $file  <<!!
$f.o:	$src/$MPFILE
	\$(CC) \$(CFLAGS) -c -o $f.o $src/$MPFILE
!!
;;
  plot) cat >> $file  <<!!
plot.o:	$src/\$(PLOTFILE)
	\$(CC) \$(CFLAGS) \$(PLOTCFLAGS) -c -o plot.o $src/\$(PLOTFILE)
!!
;;
  gp|gp_rl) cat >> $file  <<!!
$f.o:	$src/$f.c
	\$(CC) \$(CFLAGS) \$(RLINCLUDE) -c -o $f.o $src/$f.c
!!
;;
  *) cat >> $file  <<!!
$f.o:	$src/$f.c
	\$(CC) \$(CFLAGS) -c -o $f.o $src/$f.c
!!
;;
  esac
done
