/******************************************************************/
/******************************************************************/
/*                                                                */
/*                       Fichier Include PARI                     */
/*                   commun a toutes les versions                 */
/*                                                                */
/*                       copyright  Babecool                      */
/*                                                                */
/******************************************************************/
/******************************************************************/
/* $Id: gencom.h,v 1.914.7.1 1997/01/19 16:18:09 karim Exp karim $ */

#ifdef LONG_IS_64BIT
#  define MAXULONG     0xffffffffffffffffUL
#  define MAXHALFULONG 0x00000000ffffffffUL
#  define MAXUBYTE     0x000000000000ffffUL
#  define HIGHBIT      0x8000000000000000UL
#  define HIGHBITM1    0x7fffffffffffffffUL
#  define HIGHMASK     0xffffffff00000000UL
#  define LOWMASK      0x00000000ffffffffUL

#  define DEFAULTPREC     3 
#  define MEDDEFAULTPREC  4
#  define BIGDEFAULTPREC  5

#  define TWOPOTBYTES_IN_LONG  3
#  define TWOPOTBITS_IN_LONG   6
#  define BYTES_IN_LONG        8
#  define BITS_IN_LONG        64
#  define BITS_IN_HALFULONG   32
/* If you prefer a 64-bit random generator, change the following 32 to 64 */
#  define BITS_IN_RANDOM      32

#else

#  define MAXULONG     0xffffffffUL
#  define MAXHALFULONG 0x0000ffffUL
#  define MAXUBYTE     0x000000ffUL
#  define HIGHBIT      0x80000000UL
#  define HIGHBITM1    0x7fffffffUL
#  define HIGHMASK     0xffff0000UL
#  define LOWMASK      0x0000ffffUL

#  define DEFAULTPREC        4
#  define MEDDEFAULTPREC     6
#  define BIGDEFAULTPREC     8

#  define TWOPOTBYTES_IN_LONG  2
#  define TWOPOTBITS_IN_LONG   5
#  define BYTES_IN_LONG        4
#  define BITS_IN_LONG        32
#  define BITS_IN_HALFULONG   16
#  define BITS_IN_RANDOM      32
#endif

#ifndef LONG_IS_64BIT
/*  second codeword x[1], for types: 1,2,7,10,11 */
#   define EXPOBITS    0x00ffffffUL
#   define HIGHEXPOBIT 0x00800000L
#   define LGEFBITS    0x0000ffffUL
#   define VALPBITS    0x0000ffffUL /* used only for type 7 */
#   define HIGHVALPBIT 0x00008000L /* used only for type 7 */
#   define PRECPBITS   0xffff0000UL /* used only for type 7 */
#   define PRECPSHIFT  16
#   define VARNSHIFT   16
 
# ifndef OLD_CODES
#   define SIGNBITS    0xc0000000UL
#   define VARNBITS    0x3fff0000UL
#   define LGEFINTBITS 0x00ffffffUL
#   define SIGNSHIFT   30
#   define MAXVAR      16382
#   define MAXVARN     16383
# else
#   define SIGNBITS    0xff000000UL
#   define VARNBITS    0x00ff0000UL
#   define LGEFINTBITS 0x0000ffffUL
#   define SIGNSHIFT   24
#   define MAXVAR      254
#   define MAXVARN     255
# endif
 
/*  first codeword x[0] */
# ifndef OLD_CODES
#   define TYPBITS    0xfc000000UL
#   define PEREBITS   0x03000000UL
#   define LGBITS     0x00ffffffUL 
#   define TYPSHIFT   26
#   define PERESHIFT  24
#   define MAXPERE    3
# else
#   define TYPBITS    0xff000000UL
#   define PEREBITS   0x00ff0000UL
#   define LGBITS     0x0000ffffUL
#   define TYPSHIFT   24
#   define PERESHIFT  16
#   define MAXPERE    MAXUBYTE
# endif
#   define MAXLEN     LGBITS
#endif

#ifdef LONG_IS_64BIT
/*  first codeword x[0] */
#  define TYPBITS      0xffff000000000000UL
#  define PEREBITS     0x0000ffff00000000UL
#  define LGBITS       0x00000000ffffffffUL
#  define TYPSHIFT     48
#  define PERESHIFT    32
#  define MAXPERE      0xffffUL
#  define MAXLEN       LGBITS

/*  second codeword x[1] */
#  define SIGNBITS     0xffff000000000000UL
#  define VARNBITS     0x0000ffff00000000UL
#  define LGEFBITS     0x00000000ffffffffUL
#  define SIGNSHIFT    48
#  define MAXVAR       65534
#  define MAXVARN      65535

#  define EXPOBITS     0x0000ffffffffffffUL
#  define HIGHEXPOBIT  0x0000800000000000L
#  define LGEFINTBITS  0x00000000ffffffffUL
#  define VALPBITS     0x00000000ffffffffUL
#  define HIGHVALPBIT  0x0000000080000000L
#  define PRECPBITS    0xffffffff00000000UL
#  define PRECPSHIFT   32
#  define VARNSHIFT    32
#endif

/* If compiler is broken, put ((a >> BITS_IN_HALFULONG) & LOWMASK) instead */
#define HIGHWORD(a) ((a) >> BITS_IN_HALFULONG)
#define LOWWORD(a) ((a) & LOWMASK)

typedef long *GEN;
typedef unsigned long uLong;
typedef unsigned char *byteptr;

typedef struct entree {
  char *name;
  uLong valence;
  void *value;
  long menu;
  char *code;
  struct entree *next;
  char *help;
  GEN *args;
} entree;

typedef struct default_type
{
  char *name;
  void *fun;
} default_type;

typedef struct PariOUT {
  void (*putc)(char);
  void (*puts)(char*);
} PariOUT;

typedef struct PariERR {
  void (*putc)(char);
  void (*puts)(char*);
  void (*flush)();
  void (*die)();
} PariERR;

typedef struct outString {
  char *string;
  uLong len,size;
} outString;

#ifdef HAS_XMALLOC
# define xmalloc myxmalloc
# define xrealloc myxrealloc
#endif

#define init pari_init /* A VIRER */
#define gen2str GENtostring 

#ifdef MALLOC_PROCS
  typedef struct Malloc_procs {
    void * (*MyMalloc)(size_t);
    void * (*MyRealloc)(void*, size_t);
    void (*MyFree)(void*);
  } Malloc_procs;
#  define malloc (*malloc_procs.MyMalloc)
#  define realloc (*malloc_procs.MyRealloc)
#  define free (*malloc_procs.MyFree)
#endif

/*  Variables communes : */

#ifndef FASTASM
  extern  uLong overflow,hiremainder;
#endif

extern GEN    (*gp_history_fun)(long, long, char *, char *);

extern long   pari_randseed, DEBUGLEVEL;
extern long   prec,precdl,defaultpadicprecision;
extern long   paribuffsize;
extern long   *ordvar;
extern uLong  avma,bot,top,init_opts;
extern GEN    bernzone,gpi,geuler;
extern GEN    polvar,*polun,*polx,primetab;
extern GEN    gun,gdeux,ghalf,gi,gzero;
extern GEN    RAVYZARC;

/* Variables contenant la liste des fonctions PARI */
typedef struct module { entree *func; char **help; } module;
extern int     functions_tblsz;     /* taille de la table de hashcodes */
extern module  *pari_modules;       /* liste des modules de fonctions */
extern entree  **functions_hash;    /* table de hachage */
extern char    *helpmessages_basic[];
extern entree  functions_basic[];
extern entree  **varentries;
extern char    *errmessage[]; 

/* Variables contenant la liste des fonctions specifiques a GP */
extern char    *helpmessages_gp[];
extern entree   functions_gp[];

/* Variables contenant la liste des vieilles fonctions PARI (jusqu'a 1.39) */
extern module  *pari_oldmodules;    /* liste des modules de fonctions */
extern entree  **funct_old_hash;    /* table de hachage */
extern char    *oldhelpmessage[], *helpmessages_oldgp[];
extern entree   oldfonctions[], functions_oldgp[];

extern char    current_logfile[128],current_psfile[128],*current_function;
extern jmp_buf environnement;
extern FILE    *outfile, *logfile, *infile, *errfile;
extern PariOUT *pariOut;
extern PariERR *pariErr;

extern byteptr diffptr;

#define NUMPRTBELT     100  /* taille table de premiers prives */

#define INIT_JMPm 1
#define INIT_SIGm 2
#define INIT_JMP     (init_opts & INIT_JMPm)
#define INIT_SIG     (init_opts & INIT_SIGm)
#define INIT_JMP_on  (init_opts |= INIT_JMPm)
#define INIT_SIG_on  (init_opts |= INIT_SIGm)
#define INIT_JMP_off (init_opts &= ~INIT_JMPm)
#define INIT_SIG_off (init_opts &= ~INIT_SIGm)

extern const double K;        /* 32*log(2)/log(10)    */
extern const double K1;       /* log(10)/(32*log(2))  */
extern const double K2;       /* 1/(1-(log(2)/(2*pi)))*/
extern const double K4;       /* e*pi/16              */
extern const double LOG2;     /* log(2)               */
extern const double L2SL10;   /* log(2)/log(10)       */
extern const double C1;       /* log(2*pi)/2          */
extern const double C2;       /* 32*log(2)            */
extern const double C3;       /* log((1+sqrt(5))/2)/(32*log(2)) */
extern const double C31;      /* 2^31                 */
extern const long BIGINT;     /* 2^15-1               */
extern const long EXP220;     /* 2^20                 */
extern const long VERYBIGINT; /* 2^31-1               */
#ifndef  PI
  extern const double PI;
#endif

extern const long lontyp[];

extern void *foreignHandler; /* Handler for foreign commands. */
extern GEN  (*foreignExprHandler)(char*); /* Handler for foreign expressions*/
extern char foreignExprSwitch; /* Just some unprobable char. */
extern long (*foreignAutoload)(char*, long); /* What to call if unknown
					        function is found. */
extern void (*foreignFuncFree)(entree *); /* How to free external entree. */

#ifdef __cplusplus
#  define ANYARG ...
#  define BEGINEXTERN extern "C" {
#  define ENDEXTERN }
#  define INLINE inline
#  ifdef __GNUC__
#    define VOLATILE volatile
#  else
#    define VOLATILE
#  endif
#else
#  define ANYARG
#  define BEGINEXTERN
#  define ENDEXTERN
#  ifdef __GNUC__
#    define INLINE __inline__ extern
#    define VOLATILE __volatile__
#  else
#    define VOLATILE
#  endif
#endif

#ifdef NOEXP2
#  ifdef __cplusplus
     inline double exp2(double x) {return exp(x*LOG2);}
     inline double log2(double x) {return log(x)/LOG2;}
#  else
#    define exp2(x) (exp((double)(x)*LOG2))
#    define log2(x) (log((double)(x))/LOG2)
#  endif
#else
  BEGINEXTERN
    double exp2(double);
    double log2(double);
  ENDEXTERN
#endif

#ifndef LONG_IS_64BIT
#  undef labs
#  define labs(x) abs(x)
#endif

#define min(a,b) ((a)>(b)?(b):(a))
#define max(a,b) ((a)>(b)?(a):(b))

#define output(x)   {bruteall(x,'g',-1,1); pariputc('\n'); fflush(stdout);}
#define outmat(x)   {matbrute(x,'g',-1); pariputc('\n'); fflush(stdout);}
#define outbeaut(x) {sor(x,'g',-1,0); pariputc('\n'); fflush(stdout);}

#define gval(x,v) ggval(x,polx[v])
#define gvar9(x) ((typ(x)==t_POLMOD) ? gvar2(x) : gvar(x))

#define ggrando(x,n) grando0(x,n,1)
#define ggrandocp(x,n) grando0(x,n,0)

#define labsi   (long)absi
#define labsr   (long)absr
#define lach    (long)gach
#define lacos   (long)gacos
#define ladd    (long)gadd
#define laddii  (long)addii
#define laddir  (long)addir
#define laddis  (long)addis
#define laddmat (long)gaddmat
#define laddrr  (long)addrr
#define laddrs  (long)addrs
#define laddsi  (long)addsi
#define laddsmat (long)gaddsmat
#define laddsr  (long)addsr
#define ladj    (long)adj
#define larg    (long)garg
#define lash    (long)gash
#define lasin   (long)gasin
#define lassmat (long)assmat
#define latan   (long)gatan
#define lath    (long)gath
#define lbezout (long)bezout
#define lbinome (long)binome
#define lcaract (long)caract
#define lcaradj (long)caradj
#define lceil   (long)gceil
#define lch     (long)gch
#define lchangevar (long)changevar
#define lclone  (long)gclone
#define lco8    (long)co8
#define lconcat (long)concat
#define lconj   (long)gconj
#define lcontent (long)content
#define lcopy   (long)gcopy
#define lcos    (long)gcos
#define lcvtoi  (long)gcvtoi
#define lderiv  (long)deriv
#define ldet    (long)det
#define ldet2   (long)det2
#define ldeuc   (long)gdeuc
#define ldiscsr (long)discsr
#define ldiv    (long)gdiv
#define ldivii  (long)divii
#define ldivir  (long)divir
#define ldivis  (long)divis
#define ldivmod (long)gdivmod
#define ldivres (long)poldivres
#define ldivri  (long)divri
#define ldivrr  (long)divrr
#define ldivrs  (long)divrs
#define ldivsi  (long)divsi
#define ldivsr  (long)divsr
#define ldvmdii (long)dvmdii
#define ldvmdis (long)dvmdis
#define ldvmdsi (long)dvmdsi
#define lexp    (long)gexp
#define lfibo   (long)fibo
#define lfloor  (long)gfloor
#define lfrac   (long)gfrac
#define lgamd   (long)ggamd
#define lgamma  (long)ggamma
#define lgauss  (long)gauss
#define lgcd    (long)ggcd
#define lgen    (long)getgen
#define lgetg   (long)cgetg
#define lgeti   (long)cgeti
#define lgetp   (long)cgetp
#define lgetr   (long)cgetr
#define lgreffe (long)greffe
#define lhilb   (long)hilb
#define limag   (long)gimag
#define linteg  (long)integ
#define linv    (long)ginv
#define linvmat (long)invmat
#define linvmod (long)ginvmod
#define linvmulmat (long)invmulmat
#define llegendre (long)legendre
#define llift   (long)lift
#define llngamma  (long)glngamma
#define llog    (long)glog
#define lmax    (long)gmax
#define lmin    (long)gmin
#define lmod    (long)gmod
#define lmodii  (long)modii
#define lmodulcp  (long)gmodulcp
#define lmodulo  (long)gmodulo
#define lmpabs  (long)mpabs
#define lmpach  (long)mpach
#define lmpacos (long)mpacos
#define lmpadd  (long)mpadd
#define lmpash  (long)mpash
#define lmpasin (long)mpasin
#define lmpatan (long)mpatan
#define lmpath  (long)mpath
#define lmpaut  (long)mpaut
#define lmpch   (long)mpch
#define lmpcos  (long)mpcos
#define lmpdiv  (long)mpdiv
#define lmpent  (long)mpent
#define lmpeuler (long)mpeuler
#define lmpexp  (long)mpexp
#define lmpexp1 (long)mpexp1
#define lmpfact (long)mpfact
#define lmpgamd (long)mpgamd
#define lmpgamma (long)mpgamma
#define lmpinvmod (long)mpinvmod
#define lmplngamma (long)mplngamma
#define lmplog  (long)mplog
#define lmpmul  (long)mpmul
#define lmpneg  (long)mpneg
#define lmppgcd (long)mppgcd
#define lmppi   (long)mppi
#define lmppsi  (long)mppsi
#define lmpsc1  (long)mpsc1
#define lmpsh   (long)mpsh
#define lmpshift (long)mpshift
#define lmpsin  (long)mpsin
#define lmpsqrt (long)mpsqrt
#define lmpsub  (long)mpsub
#define lmptan  (long)mptan
#define lmpth   (long)mpth
#define lmptrunc (long)mptrunc
#define lmul    (long)gmul
#define lmul2n  (long)gmul2n
#define lmulii  (long)mulii
#define lmulir  (long)mulir
#define lmulis  (long)mulis
#define lmulri  (long)mulri
#define lmulrr  (long)mulrr
#define lmulrs  (long)mulrs
#define lmulsi  (long)mulsi
#define lmulsr  (long)mulsr
#define lmulss  (long)mulss
#define lneg    (long)gneg
#define lnegi   (long)negi
#define lnegr   (long)negr
#define lnorm   (long)gnorm
#define lnorml2 (long)gnorml2
#define lopgs2  (long)gopgs2
#define lopsg2  (long)gopsg2
#define lpasc   (long)pasc
#define lpile   (long)gerepile
#define lpoleval (long)poleval
#define lpolgcd (long)polgcd
#define lprec   (long)gprec
#define lprimpart (long)primpart
#define lpsi    (long)gpsi
#define lpui    (long)gpui
#define lpuigs  (long)gpuigs
#define lpuissmodulo (long)puissmodulo
#define lquadgen (long)quadgen
#define lquadpoly (long)quadpoly
#define lracine (long)racine
#define lreal   (long)greal
#define lrecip  (long)recip
#define lred    (long)gred
#define lres    (long)gres
#define lresii  (long)resii
#define lrndtoi (long)grndtoi
#define lroots  (long)roots
#define lround  (long)ground
#define lscalmat (long)gscalmat
#define lscalsmat (long)gscalsmat
#define lsfcont (long)sfcont
#define lsh     (long)gsh
#define lshift  (long)gshift
#define lshifti (long)shifti
#define lshiftr (long)shiftr
#define lsin    (long)gsin
#define lsqr    (long)gsqr
#define lsqri   (long)sqri
#define lsqrt   (long)gsqrt
#define lstoi   (long)stoi
#define lsub    (long)gsub
#define lsubii  (long)subii
#define lsubir  (long)subir
#define lsubis  (long)subis
#define lsubres (long)subres
#define lsubri  (long)subri
#define lsubrr  (long)subrr
#define lsubrs  (long)subrs
#define lsubsi  (long)subsi
#define lsubsr  (long)subsr
#define lsubst  (long)gsubst
#define ltan    (long)gtan
#define ltchebi (long)tchebi
#define lth     (long)gth
#define ltrace  (long)trace
#define ltrans  (long)gtrans
#define ltree   (long)gettree
#define ltrunc  (long)gtrunc

#define zero    (long)gzero
#define un      (long)gun
#define deux    (long)gdeux
#define lhalf   (long)ghalf
#define lpolx   (long)polx
#define lpolun  (long)polun

#define addis(x,s)  addsi(s,x)
#define addrs(x,s)  addsr(s,x)
#define mulis(x,s)  mulsi(s,x)
#define mulri(x,s)  mulir(s,x)
#define mulrs(x,s)  mulsr(s,x)

#define mpmodz(x,y,z) (modiiz(x,y,z))
#define mpresz(x,y,z) (resiiz(x,y,z))
#define mpmod(x,y)    (modii(x,y))
#define mpres(x,y)    (resii(x,y))

#define ldivgs(x,s)   ((long)gdivgs(x,s))
#define lmulgs(y,s)   ((long)gmulsg(s,y))
#define lmulsg(s,y)   ((long)gmulsg(s,y))

#define laddgs(y,s)    (lopsg2(gadd,s,y))
#define laddsg(s,y)    (lopsg2(gadd,s,y))
#define ldiventgs(y,s) (lopgs2(gdivent,y,s))
#define ldiventsg(s,y) (lopsg2(gdivent,s,y))
#define ldivsg(s,y)    (lopsg2(gdiv,s,y))
#define lmaxgs(y,s)    (lopgs2(gmax,y,s))
#define lmaxsg(s,y)    (lopsg2(gmax,s,y))
#define lmings(y,s)    (lopgs2(gmin,y,s))
#define lminsg(s,y)    (lopsg2(gmin,s,y))
#define lmodgs(y,s)    (lopgs2(gmod,y,s))
#define lmodsg(s,y)    (lopsg2(gmod,s,y))
#define lsubgs(y,s)    (lopgs2(gsub,y,s))
#define lsubsg(s,y)    (lopsg2(gsub,s,y))

#define mppiz(x)       (gop0z(mppi,x))
#define mpeulerz(x)    (gop0z(mpeuler,x))

#define autz(x,y)      (gop1z(mpaut,x,y))
#define mpsqrtz(x,y)   (gop1z(mpsqrt,x,y))
#define mpexpz(x,y)    (gop1z(mpexp,x,y))
#define mpexp1z(x,y)   (gop1z(mpexp1,x,y))
#define mplogz(x,y)    (gop1z(mplog,x,y))
#define mpcosz(x,y)    (gop1z(mpcos,x,y))
#define mpsinz(x,y)    (gop1z(mpsin,x,y))
#define mptanz(x,y)    (gop1z(mptan,x,y))
#define mpatanz(x,y)   (gop1z(mpatan,x,y))
#define mpasinz(x,y)   (gop1z(mpasin,x,y))
#define mpacosz(x,y)   (gop1z(mpacos,x,y))
#define mpchz(x,y)     (gop1z(mpch,x,y))
#define mpshz(x,y)     (gop1z(mpsh,x,y))
#define mpthz(x,y)     (gop1z(mpth,x,y))
#define mpathz(x,y)    (gop1z(mpath,x,y))
#define mpashz(x,y)    (gop1z(mpash,x,y))
#define mpachz(x,y)    (gop1z(mpach,x,y))
#define mpgammaz(x,y)  (gop1z(mpgamma,x,y))
#define gredz(x,y)     (gop1z(gred,x,y))
#define gnegz(x,y)     (gop1z(gneg,x,y))

#define mpfactz(s,y)   (gops1z(mpfact,s,y))

#define mpargz(x,y,z)   (gop2z(mparg,x,y,z))
#define gabsz(x,prec,y) (gop2z(gabs,x,prec,y))
#define gmaxz(x,y,z)    (gop2z(gmax,x,y,z))
#define gminz(x,y,z)    (gop2z(gmin,x,y,z))
#define gaddz(x,y,z)    (gop2z(gadd,x,y,z))
#define gsubz(x,y,z)    (gop2z(gsub,x,y,z))
#define gmulz(x,y,z)    (gop2z(gmul,x,y,z))
#define gdivz(x,y,z)    (gop2z(gdiv,x,y,z))
#define gdeucz(x,y,z)   (gop2z(gdeuc,x,y,z))
#define gdiventz(x,y,z) (gop2z(gdivent,x,y,z))
#define gmodz(x,y,z)    (gop2z(gmod,x,y,z))

#define gaddgs(y,s)     (gopsg2(gadd,s,y))
#define gaddsg(s,y)     (gopsg2(gadd,s,y))
#define gdiventsg(s,y)  (gopsg2(gdivent,s,y))
#define gdivsg(s,y)     (gopsg2(gdiv,s,y))
#define gmaxsg(s,y)     (gopsg2(gmax,s,y))
#define gminsg(s,y)     (gopsg2(gmin,s,y))
#define gmodsg(s,y)     (gopsg2(gmod,s,y))
#define gsubsg(s,y)     (gopsg2(gsub,s,y))

#define gdiventgs(y,s)  (gopgs2(gdivent,y,s))
#define gmaxgs(y,s)     (gopgs2(gmax,y,s))
#define gmings(y,s)     (gopgs2(gmin,y,s))
#define gmodgs(y,s)     (gopgs2(gmod,y,s))
#define gsubgs(y,s)     (gopgs2(gsub,y,s))

#define gcmpsg(s,y)     (-opgs2(gcmp,y,s))
#define gcmpgs(y,s)     (opgs2(gcmp,y,s))
#define gegalsg(s,y)    (opgs2(gegal,y,s))
#define gegalgs(y,s)    (opgs2(gegal,y,s))

#define gaddgsz(y,s,z)    (gopsg2z(gadd,s,y,z))
#define gaddsgz(s,y,z)    (gopsg2z(gadd,s,y,z))
#define gdiventsgz(s,y,z) (gopsg2z(gdivent,s,y,z))
#define gdivsgz(s,y,z)    (gopsg2z(gdiv,s,y,z))
#define gmaxsgz(s,y,z)    (gopsg2z(gmax,s,y,z))
#define gminsgz(s,y,z)    (gopsg2z(gmin,s,y,z))
#define gmodsgz(s,y,z)    (gopsg2z(gmod,s,y,z))
#define gsubsgz(s,y,z)    (gopsg2z(gsub,s,y,z))

#define gdiventgsz(y,s,z) (gopgs2z(gdivent,y,s,z))
#define gmaxgsz(y,s,z)    (gopgs2z(gmax,y,s,z))
#define gmingsz(y,s,z)    (gopgs2z(gmin,y,s,z))
#define gmodgsz(y,s,z)    (gopgs2z(gmod,y,s,z))
#define gsubgsz(y,s,z)    (gopgs2z(gsub,y,s,z))

#define gdivgsz(y,s,z)  (gops2gsz(gdivgs,y,s,z))
#define gmul2nz(x,s,z)  (gops2gsz(gmul2n,x,s,z))
#define gmulgsz(y,s,z)  (gops2sgz(gmulsg,s,y,z))
#define gmulsgz(s,y,z)  (gops2sgz(gmulsg,s,y,z))
#define gshiftz(x,s,z)  (gops2gsz(gshift,x,s,z))

#define gmulgs(y,s)     (gmulsg(s,y))

#define mael(ma,x1,x2) ( ((GEN) ( (GEN) ma)[x1]) [x2])
#define mael2 mael
#define mael3(ma,x1,x2,x3) ( ((GEN) mael2(ma,x1,x2)) [x3])
#define mael4(ma,x1,x2,x3,x4) ( ((GEN) mael3(ma,x1,x2,x3)) [x4])
#define mael5(ma,x1,x2,x3,x4,x5) ( ((GEN) mael4(ma,x1,x2,x3,x4)) [x5])

#define gmael (GEN) mael
#define gmael2 (GEN) mael
#define gmael3 (GEN) mael3
#define gmael4 (GEN) mael4
#define gmael5 (GEN) mael5

#define coeff(a,i,j)  ( ( (GEN) ( (GEN) a)[j]) [i])
#define gcoeff(a,i,j) (GEN)coeff(a,i,j)

#define bern(i)       (GEN)(bernzone + (i)*(*(bernzone + 2)) + 3) 

#define copyifstack(x) (RAVYZARC=(GEN)(x),((RAVYZARC>=(GEN)bot)&&(RAVYZARC<(GEN)top))?lcopy(RAVYZARC):(long)RAVYZARC)

/* #define copyifstack(x) (lcopy((GEN)(x))) */
 
#ifdef __cplusplus
  inline int isonstack(GEN x)
  {
    GEN RAVYZARC=x;
    return ((RAVYZARC>=(GEN)bot)&&(RAVYZARC<(GEN)top));
  }
#else
# define isonstack(x) (\
    RAVYZARC=(GEN)(x),((RAVYZARC>=(GEN)bot)&&(RAVYZARC<(GEN)top))\
  )
#endif

#define leadingterm(x) (is_scalar_t(typ(x)) ? (GEN)x \
                                            : ((GEN)(((GEN)x)[lgef(x)-1])))

/* works only for POSITIVE integers */
#define mod64(x)  (((x)[lgefint(x)-1]) & 63)
#define mod32(x)  (((x)[lgefint(x)-1]) & 31)
#define mod16(x)  (((x)[lgefint(x)-1]) & 15)
#define mod8(x)   (((x)[lgefint(x)-1]) & 7)
#define mod4(x)   (((x)[lgefint(x)-1]) & 3)
#define mod2(x)   (((x)[lgefint(x)-1]) & 1)
#define is_pm1(n) ((((GEN)(n))[2]==1) && (lgefint(n)==3))
#define is_not_pm1(n) ((((GEN)(n))[2]!=1) || (lgefint(n)!=3))

#ifdef __cplusplus
   inline int odd(long x) {return x&1;}
#else
#  define odd(x) ((x) & 1)
#endif

#define mpodd(x) (signe(x) && mod2(x))

#define evalsigne(x)   (((long)(x)) << SIGNSHIFT)
#define evaltyp(x)     (((uLong)(x)) << TYPSHIFT)
#define evalpere(x)    (((uLong)(x)) << PERESHIFT)
#define evallg(x)      (x)
#define evallgef(x)    (x)
#define evallgefint(x) (x)
#define evalvarn(x)    (((uLong)(x)) << VARNSHIFT)
#define evalprecp(x)   (((long)(x)) << PRECPSHIFT)
#define evalexpo(x)    (HIGHEXPOBIT + (x))
#define evalvalp(x)    (HIGHVALPBIT + (x))

/* alglin1.c */

GEN     concat(GEN x, GEN y);
GEN     deplin(GEN x);
GEN     det(GEN a);
GEN     det0(GEN a,long flag);
GEN     det2(GEN a);
GEN     dethnf(GEN x);
GEN     detint(GEN x);
GEN     detreel(GEN a);
GEN     diagonal(GEN x);
GEN     eigen(GEN x, long prec);
GEN     extract(GEN x, GEN l);
GEN     gaddmat(GEN x, GEN y);
GEN     gaddsmat(long s, GEN y);
GEN     gauss(GEN a, GEN b);
GEN     gaussmodulo(GEN M, GEN D, GEN Y);
GEN     gaussmodulo0(GEN M, GEN D, GEN Y,long flag);
GEN     gaussmodulo2(GEN M, GEN D, GEN Y);
GEN     gscalmat(GEN x, long n);
GEN     gscalsmat(long x, long n);
GEN     gtomat(GEN x);
GEN     gtrans(GEN x);
GEN     hnfadd(GEN mit,GEN* ptpdep,GEN* ptmatc,long* vperm,GEN* ptmatalpha, long co,long li,long col,long* ptnlze,GEN extramat,GEN extramatc);
GEN     hnfspec(long** mat,GEN* ptdep,GEN* ptmatc,long* vperm,GEN* ptmatalpha, long co,long li,long k0,long* ptnlze,long* ptcol);
GEN     idmat(long n);
GEN     image(GEN x);
GEN     image0(GEN x,long flag);
GEN     image2(GEN x);
GEN     imagecompl(GEN x);
GEN     imagereel(GEN x, long prec);
GEN     indexrank(GEN x);
GEN     inverseimage(GEN mat, GEN y);
long    isdiagonal(GEN x);
GEN     ker(GEN x);
GEN     ker0(GEN x, long flag);
GEN     keri(GEN x);
GEN     matextract(GEN x, GEN l1, GEN l2);
long    rank(GEN x);
GEN     suppl(GEN x);

/* alglin2.c */

GEN     adj(GEN x);
GEN     assmat(GEN x);
GEN     caract(GEN x, int v);
GEN     caradj(GEN x, long v, GEN *py);
GEN     caradj0(GEN x, long v);
GEN     carhess(GEN x, long v);
GEN     char0(GEN x, int v,long flag);
GEN     conjvec(GEN x,long prec);
GEN     gconj(GEN x);
GEN     gnorm(GEN x);
GEN     gnorml2(GEN x);
GEN     hess(GEN x);
GEN     intersect(GEN x, GEN y);
GEN     invmat(GEN a);
GEN     invmatreel(GEN a);
GEN     invmulmat(GEN a, GEN b);
GEN     invmulmatreel(GEN a, GEN b);
GEN     jacobi(GEN a, long prec);
GEN     matrixqz(GEN x, GEN pp);
GEN     matrixqz0(GEN x, GEN pp);
GEN     matrixqz2(GEN x);
GEN     matrixqz3(GEN x);
GEN     signat(GEN a);
GEN     sqred(GEN a);
GEN     sqred1(GEN a);
GEN     sqred3(GEN a);
GEN     trace(GEN x);

/* anal.c */

long    delete_var();
long    fetch_var();
GEN     flisexpr(char *t);
void    freeep(entree *ep);
long    hashvalue(char **u, int tblsz, int do_update);
GEN     lisexpr(char *t);
GEN     lisseq(char *t);
long    manage_var(long n, entree *ep);
GEN     readexpr(char **c);
GEN     readseq(char *c);
char *  readstring(char **src, char *s, char *lim, long flag);

/* arith1.c */

GEN     bestappr(GEN x, GEN k);
GEN     bezout(GEN a, GEN b, GEN *u, GEN *v);
GEN     binaire(GEN x);
long    bittest(GEN x, long n);
long    carrecomplet(GEN x, GEN *pt);
long    carreparfait(GEN x);
GEN     chinois(GEN x, GEN y);
GEN     classno(GEN x);
GEN     qfbclassno0(GEN x,long flag);
GEN     classno2(GEN x);
GEN     classno3(GEN x);
GEN     fibo(long n);
GEN     fundunit(GEN x);
GEN     gbittest(GEN x, GEN n);
GEN     gboundcf(GEN x, long k);
GEN     gcarrecomplet(GEN x, GEN *pt);
GEN     gcarreparfait(GEN x);
GEN     gcf(GEN x);
GEN     gcf2(GEN b, GEN x);
GEN     gener(GEN m);
GEN     gfundunit(GEN x);
GEN     ggener(GEN m);
GEN     gisfundamental(GEN x);
GEN     gisprime(GEN x);
GEN     gispsp(GEN x);
GEN     gissquarefree(GEN x);
GEN     gkrogs(GEN x, long y);
GEN     gkronecker(GEN x, GEN y);
GEN     gmillerrabin(GEN n, long k);
GEN     gnextprime(GEN n);
GEN     gprecprime(GEN n);
GEN     gpseudopremier(GEN n, GEN a);
GEN     gracine(GEN a);
GEN     gregula(GEN x, long prec);
long    hil(GEN x, GEN y, GEN p);
long    hil0(GEN x, GEN y, GEN p);
int     inversemodulo(GEN a, GEN b, GEN *res);
long    isfundamental(GEN x);
long    isprime(GEN x);
long    ispsp(GEN x);
long    issquarefree(GEN x);
long    krogs(GEN x, long y);
long    kronecker(GEN x, GEN y);
long    krosg(long s, GEN x);
long    kross(long x, long y);
void    lucas(long n, GEN *ln, GEN *ln1);
long    millerrabin(GEN n, long k);
GEN     mpfact(long n);
GEN     mpfactr(long n, long prec);
GEN     mpinvmod(GEN a, GEN m);
GEN     mppgcd(GEN a, GEN b);
int     mpsqrtmod(GEN a, GEN p, GEN *pr);
GEN     nextprime(GEN n);
GEN     order(GEN x);
GEN     pnqn(GEN x);
GEN     precprime(GEN n);
long    pseudopremier(GEN n, GEN a);
GEN     puissmodulo(GEN a, GEN n, GEN m);
GEN     quaddisc(GEN x);
GEN     racine(GEN a);
GEN     regula(GEN x, long prec);
GEN     sfcont(GEN x, GEN x1, long k);
GEN     sfcont0(GEN x, GEN b, long flag);
GEN     znstar(GEN x);

/* arith2.c */

GEN     Qfb0(GEN x, GEN y, GEN z, GEN d, long prec);
GEN     addprimestotable(GEN primes);
GEN     auxdecomp(GEN n, long all);
long    bigomega(GEN n);
GEN     boundfact(GEN n, long lim);
GEN     compimag(GEN x, GEN y);
GEN     compreal(GEN x, GEN y);
GEN     comprealraw(GEN x, GEN y);
GEN     core(GEN n);
GEN     core0(GEN n,long flag);
GEN     core2(GEN n);
GEN     coredisc(GEN n);
GEN     coredisc0(GEN n,long flag);
GEN     coredisc2(GEN n);
GEN     decomp(GEN n);
GEN     divisors(GEN n);
GEN     gbigomega(GEN n);
GEN     gboundfact(GEN n, long lim);
GEN     gmu(GEN n);
GEN     gnumbdiv(GEN n);
GEN     gomega(GEN n);
GEN     gphi(GEN n);
GEN     gsumdiv(GEN n);
GEN     gsumdivk(GEN n,long k);
byteptr initprimes(long maxnum);
long    mu(GEN n);
GEN     nucomp(GEN x, GEN y, GEN l);
GEN     nudupl(GEN x, GEN l);
GEN     numbdiv(GEN n);
GEN     nupow(GEN x, GEN n);
long    omega(GEN n);
GEN     phi(GEN n);
GEN     powrealraw(GEN x, long n, long prec);
GEN     prime(long n);
GEN     primeform(GEN x, GEN p, long prec);
GEN     primes(long n);
GEN     qfi(GEN x, GEN y, GEN z);
GEN     qfr(GEN x, GEN y, GEN z, GEN d);
GEN     qfbred0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD);
GEN     redimag(GEN x);
GEN     redreal(GEN x);
GEN     redreal0(GEN x,long flag,GEN D,GEN isqrtD, GEN sqrtD);
GEN     redrealnod(GEN x, GEN isqrtD);
GEN     rhoreal(GEN x);
GEN     rhorealnod(GEN x, GEN isqrtD);
GEN     smallfact(GEN n);
GEN     sqcomp(GEN x);
GEN     sqcompreal(GEN x);
GEN     sqcomprealraw(GEN x);
GEN     sumdiv(GEN n);
GEN     sumdivk(GEN n,long k);

/* base1.c */

void    checkbid(GEN bid);
void    checkbnf(GEN bnf);
void    checkbnr(GEN bnr);
void    checkbnrgen(GEN bnr);
GEN     checknf(GEN nf);
void 	checkrnf(GEN rnf);
GEN     galois(GEN x, long prec);
GEN     galoisapply(GEN nf, GEN aut, GEN x);
GEN     galoisconj(GEN nf);
GEN     galoisconj0(GEN nf,long flag, long prec);
GEN     galoisconj1(GEN x, long prec);
GEN     galoisconj2(GEN x, long prec);
GEN     galoisconjforce(GEN x);
GEN     glambdak(GEN nfz, GEN s, long prec);
int     gpolcomp(GEN p1, GEN p2);
GEN     gsmith(GEN x);
GEN     gsmith2(GEN x);
GEN     gzetak(GEN nfz, GEN s, long prec);
GEN     gzetakall(GEN nfz, GEN s, long flag, long prec);
GEN     hnf(GEN x);
GEN     hnf0(GEN x,long flag);
GEN     hnfall(GEN x);
GEN     hnfhavas(GEN x);
GEN     hnflll(GEN x);
GEN     hnfmod(GEN x, GEN detmat);
GEN     hnfmodid(GEN x,GEN p);
GEN     hnfperm(GEN x);
GEN     initalg(GEN x, long prec);
GEN     initalgred(GEN x, long prec);
GEN     initalgred2(GEN x, long prec);
GEN     initzeta(GEN pol, long prec);
GEN     isinclfast(GEN nf1, GEN nf2);
GEN     isisomfast(GEN nf1, GEN nf2);
GEN     isisomfastall(GEN nf1, GEN nf2, long fliso);
GEN     nfinit0(GEN x,long flag, long prec);
GEN     rootsof1(GEN x);
GEN     smith(GEN x);
GEN     smith0(GEN x,long flag);
GEN     smith2(GEN x);
GEN     smithclean(GEN z);
GEN     tschirnhaus(GEN x);

/* base2.c */

GEN     allbase(GEN x, long code, GEN *y);
GEN     allbase4(GEN f, long code, GEN *y, GEN *ptw);
GEN     base(GEN x, GEN *y);
GEN     base2(GEN x, GEN *y);
GEN     bestnu(GEN w);
GEN     compositum(GEN pol1, GEN pol2);
GEN     compositum2(GEN pol1, GEN pol2);
GEN     discf(GEN x);
GEN     discf0(GEN x,long flag, GEN p);
GEN     discf2(GEN x);
GEN     factcp(GEN p,GEN f,GEN beta);
GEN     factoredbase(GEN x, GEN p, GEN *y);
GEN     factoreddiscf(GEN x, GEN p);
GEN     gcdpm(GEN f1,GEN f2,GEN pm);
long    idealval(GEN nf,GEN ix,GEN vp);
GEN     nfbasis0(GEN x, GEN *y,long flag,GEN p);
GEN     nfreducemodpr(GEN nf, GEN x, GEN prhall);
GEN     polcompositum0(GEN pol1, GEN pol2,long flag);
GEN     primedec(GEN nf,GEN p);
GEN     rnfbasis(GEN bnf, GEN order);
GEN     rnfdet(GEN nf, GEN order);
GEN     rnfdet0(GEN nf, GEN x, GEN y);
GEN     rnfdet2(GEN nf, GEN A, GEN I);
GEN     rnfdiscf(GEN nf, GEN pol);
GEN     rnfequation(GEN nf, GEN pol2);
GEN     rnfequation2(GEN nf, GEN pol);
GEN     rnfequationall(GEN nf, GEN pol2, long flall);
GEN     rnfhermitebasis(GEN bnf, GEN order);
long    rnfisfree(GEN bnf, GEN order);
GEN     rnflllgram(GEN nf, GEN pol, GEN order,long prec);
GEN     rnfpolred(GEN nf, GEN pol, long prec);
GEN     rnfpseudobasis(GEN nf, GEN pol);
GEN     rnfsimplifybasis(GEN bnf, GEN order);
GEN     rnfsteinitz(GEN nf, GEN order);
GEN     setup(GEN p,GEN f,GEN theta,GEN nut);
GEN     smallbase(GEN x, GEN *y);
GEN     smalldiscf(GEN x);
GEN     subcyclo(GEN p, GEN d, int n);

/* base3.c */

#define element_divmodideal(nf,x,y,ideal) (\
  nfreducemodideal(nf,\
    element_mul(nf,x,element_div(nf,(GEN)idealaddone(nf,y,ideal)[1],y))\
    ,ideal\
  )\
)
#define element_mulmodideal(nf,x,y,ideal) (\
  nfreducemodideal(nf,element_mul(nf,x,y),ideal)\
)
#define element_mulmodidele(nf,x,y,idele,structarch) (\
  nfreducemodidele(nf,element_mul(nf,x,y),idele,structarch)\
  )
#define element_mulmodpr(nf,x,y,prhall) (\
  nfreducemodpr(nf,element_mul(nf,x,y),prhall)\
)
#define element_mulmodpr(nf,x,y,prhall) (\
  nfreducemodpr(nf,element_mul(nf,x,y),prhall)\
)
#define element_sqrmodideal(nf,x,ideal) (\
  nfreducemodideal(nf,element_sqr(nf,x),ideal)\
)
#define element_sqrmodidele(nf,x,idele,structarch) (\
  nfreducemodidele(nf,element_sqr(nf,x),idele,structarch)\
)
#define element_sqrmodpr(nf,x,prhall) (\
  nfreducemodpr(nf,element_sqr(nf,x),prhall)\
)
#define element_sqrmodpr(nf,x,prhall) (\
  nfreducemodpr(nf,element_sqr(nf,x),prhall)\
)
#define idealmulmodideal(nf,x,y,ideal,prec) (\
  ideallllredmodideal(nf,idealmullll(nf,x,y),ideal,prec)\
)

GEN     algtobasis(GEN nf, GEN x);
GEN     basistoalg(GEN nf, GEN x);
GEN     element_div(GEN nf, GEN x, GEN y);
GEN     element_inv(GEN nf, GEN x);
GEN     element_mul(GEN nf,GEN x,GEN y);
GEN     element_mulh(GEN nf, long limi, long limj, GEN x, GEN y);
GEN     element_mulvec(GEN nf, GEN x, GEN v);
GEN     element_pow(GEN nf,GEN x,GEN k);
GEN     element_powmodideal(GEN nf,GEN x,GEN k,GEN ideal);
GEN     element_powmodidele(GEN nf,GEN x,GEN k,GEN idele,GEN structarch);
GEN     element_sqr(GEN nf,GEN x);
long    element_val(GEN nf, GEN x, GEN vp);
long    element_val2(GEN nf, GEN x, GEN d, GEN vp);
GEN     ideallist(GEN nf,long bound);
GEN     ideallist0(GEN nf,long bound, long flag);
GEN     ideallistarch(GEN nf, GEN list, GEN arch);
GEN     ideallistarch0(GEN nf, GEN list, GEN arch,long flag);
GEN     ideallistarchgen(GEN nf, GEN list, GEN arch);
GEN     ideallistunit(GEN nf,long bound);
GEN     ideallistunitarch(GEN bnf,GEN list,GEN arch);
GEN     ideallistunitarchgen(GEN bnf,GEN list,GEN arch);
GEN     ideallistunitgen(GEN nf,long bound);
GEN     ideallistzstar(GEN nf,long bound);
GEN     ideallistzstargen(GEN nf,long bound);
GEN     lllreducemodmatrix(GEN x,GEN y);
GEN     nfdiveuc(GEN nf, GEN a, GEN b);
GEN     nfdivres(GEN nf, GEN a, GEN b);
GEN     nfmod(GEN nf, GEN a, GEN b);
GEN     nfreducemodidele(GEN nf,GEN g,GEN idele,GEN structarch);
GEN     nfshanks(GEN nf,GEN x,GEN g0,GEN pr,GEN prhall);
GEN     reducemodmatrix(GEN x, GEN y);
GEN     zarchstar(GEN nf,GEN x,GEN arch,long nba);
GEN     zideallog(GEN nf,GEN x,GEN bigideal);
GEN     zidealstar(GEN nf, GEN x);
GEN     zidealstar0(GEN nf, GEN x,long flag);
GEN     zidealstarinit(GEN nf, GEN x);
GEN     zidealstarinitall(GEN nf, GEN x,long flun);
GEN     zidealstarinitgen(GEN nf, GEN x);
GEN     zidealstarinitjoin(GEN nf, GEN bid1, GEN bid2);
GEN     zidealstarinitjoinarch(GEN nf, GEN bid1, GEN arch, long nba);
GEN     zidealstarinitjoinarchgen(GEN nf, GEN bid1, GEN arch, long nba);
GEN     zidealstarinitjoingen(GEN nf, GEN bid1, GEN bid2);
GEN     zsigne(GEN nf,GEN alpha,GEN arch,long nba);

/* base4.c */

GEN     differente(GEN nf, GEN premiers);
GEN     element_divmodpr(GEN nf, GEN x, GEN y, GEN prhall);
GEN     element_invmodpr(GEN nf, GEN y, GEN prhall);
GEN     element_powmodpr(GEN nf, GEN x, GEN k, GEN prhall);
GEN     element_reduce(GEN nf, GEN x, GEN ideal);
GEN     ideal_two_elt(GEN nf, GEN ix);
GEN     ideal_two_elt0(GEN nf, GEN ix, GEN a);
GEN     ideal_two_elt2(GEN nf, GEN x, GEN a);
GEN     idealadd(GEN nf, GEN x, GEN y);
GEN     idealaddmultone(GEN nf, GEN list);
GEN     idealaddone(GEN nf, GEN x, GEN y);
GEN     idealaddone0(GEN nf, GEN x, GEN y);
GEN     idealappr(GEN nf, GEN x);
GEN     idealapprall(GEN nf, GEN x, long fl);
GEN     idealapprfact(GEN nf, GEN x);
GEN     idealchinese(GEN nf, GEN x, GEN y);
GEN     idealcoprime(GEN nf, GEN x, GEN y);
GEN     idealdiv(GEN nf, GEN x, GEN y);
GEN     idealdiv0(GEN nf, GEN x, GEN y,long flag);
GEN     idealdivexact(GEN nf, GEN x, GEN y);
GEN     idealfactor(GEN nf, GEN x);
GEN     idealhermite(GEN nf, GEN x);
GEN     idealhermite0(GEN nf, GEN a, GEN b);
GEN     idealhermite2(GEN nf, GEN a, GEN b);
GEN     idealintersect(GEN nf, GEN x, GEN y);
GEN     idealinv(GEN nf, GEN ix);
GEN     idealinv0(GEN nf, GEN ix,long flag);
GEN     ideallllred(GEN nf,GEN ix,GEN vdir,long prec);
GEN     ideallllredall(GEN nf, GEN ix, GEN vdir, long prec, long precint);
GEN     ideallllredpart1(GEN nf,GEN x,GEN vdir, long flprem, long prec);
GEN     ideallllredpart1spec(GEN nf, GEN x, GEN matt2, long flprem, long prec);
GEN     ideallllredpart2(GEN nf,GEN arch,GEN z,long prec);
GEN     idealmul(GEN nf, GEN ix, GEN iy);
GEN     idealmul0(GEN nf, GEN ix, GEN iy, long flag, long prec);
GEN     idealmulelt(GEN nf, GEN elt, GEN x);
GEN     idealmulh(GEN nf, GEN ix, GEN iy);
GEN     idealmullll(GEN nf, GEN x, GEN y);
GEN     idealmulprime(GEN nf,GEN ix,GEN vp);
GEN     idealmulred(GEN nf, GEN ix, GEN iy, long prec);
GEN     idealnorm(GEN nf, GEN x);
GEN     idealpow(GEN nf, GEN ix, GEN n);
GEN     idealpow0(GEN nf, GEN ix, GEN n, long flag, long prec);
GEN     idealpowprime(GEN nf, GEN vp, GEN n,long prec);
GEN     idealpowred(GEN nf, GEN ix, GEN n,long prec);
GEN     idealpows(GEN nf, GEN ideal, long iexp);
GEN     ideleaddone(GEN nf, GEN x, GEN idele);
long    isideal(GEN nf,GEN x);
long    isinvector(GEN v, GEN x, long n);
GEN     minideal(GEN nf,GEN ix,GEN vdir,long prec);
GEN     nfdetint(GEN nf,GEN pseudo);
GEN     nfhermite(GEN nf, GEN x);
GEN     nfhermitemod(GEN nf, GEN x, GEN detmat);
GEN     nfsmith(GEN nf, GEN x);
GEN     oldidealinv(GEN nf, GEN ix);
GEN     principalideal(GEN nf, GEN a);
GEN     principalidele(GEN nf, GEN a);
GEN     threetotwo(GEN nf, GEN a, GEN b, GEN c);
GEN     threetotwo2(GEN nf, GEN a, GEN b, GEN c);
GEN     twototwo(GEN nf, GEN a, GEN b);

/* base5.c */

GEN     matbasistoalg(GEN nf, GEN x);
GEN     matalgtobasis(GEN nf, GEN x);
GEN     rnfalgtobasis(GEN rnf, GEN x);
GEN     rnfbasistoalg(GEN rnf, GEN x);
GEN     rnfelementabstorel(GEN rnf, GEN x);
GEN     rnfelementdown(GEN rnf, GEN x);
GEN     rnfelementreltoabs(GEN rnf, GEN x);
GEN     rnfelementup(GEN rnf, GEN x);
GEN     rnfidealabstorel(GEN rnf, GEN x);
GEN     rnfidealdown(GEN rnf, GEN x);
GEN     rnfidealhermite(GEN rnf, GEN x);
GEN     rnfidealmul(GEN rnf,GEN x,GEN y);
GEN     rnfidealnormabs(GEN rnf, GEN x);
GEN     rnfidealnormrel(GEN rnf, GEN x);
GEN     rnfidealreltoabs(GEN rnf, GEN x);
GEN     rnfidealtwoelement(GEN rnf,GEN x);
GEN     rnfidealup(GEN rnf, GEN x);
GEN     rnfinitalg(GEN nf,GEN pol,long prec);

/* bibli1.c */

GEN     algdep(GEN x, long n, long prec);
GEN     algdep0(GEN x, long n, long bit,long prec);
GEN     algdep2(GEN x, long n, long bit);
GEN     allpolredabs(GEN x, long flag, long prec);
GEN     binome(GEN x, long k);
GEN     changevar(GEN x, GEN y);
GEN     convol(GEN x, GEN y);
GEN     cyclo(long n, long v);
GEN     factoredpolred(GEN x, GEN p, long prec);
GEN     factoredpolred2(GEN x, GEN p, long prec);
GEN     fincke_pohst(GEN a, GEN borne, GEN stockmax, long prec);
GEN     gconvpe(GEN x);
GEN     gconvsp(GEN x);
GEN     gprec(GEN x, long l);
GEN     grando0(GEN x, long n, long do_clone);
GEN     hilb(long n);
GEN     indexlexsort(GEN x);
GEN     indexsort(GEN x);
GEN     kerint(GEN x);
GEN     kerint0(GEN x,long flag);
GEN     kerint1(GEN x);
GEN     kerint2(GEN x);
GEN     laplace(GEN x);
GEN     legendre(long n, long v);
GEN     lexsort(GEN x);
GEN     lindep(GEN x, long prec);
GEN     lindep0(GEN x, long flag,long prec);
GEN     lindep2(GEN x, long bit);
GEN     lll(GEN x, long prec);
GEN     lll0(GEN x, long flag, long prec);
GEN     lll1(GEN x, long prec);
GEN     lllgen(GEN x);
GEN     lllgram(GEN x, long prec);
GEN     lllgram0(GEN x, long flag,long prec);
GEN     lllgram1(GEN x, long prec);
GEN     lllgramgen(GEN x);
GEN     lllgramint(GEN x);
GEN     lllgramkerim(GEN x);
GEN     lllgramkerimgen(GEN x);
GEN     lllint(GEN x);
GEN     lllintpartial(GEN mat);
GEN     lllkerim(GEN x);
GEN     lllkerimgen(GEN x);
GEN     lllrat(GEN x);
GEN     minim(GEN a, GEN borne, GEN stockmax);
GEN     minim0(GEN a, GEN borne, GEN stockmax,long flag, long prec);
GEN     minim2(GEN a, GEN borne, GEN stockmax);
GEN     ordred(GEN x, long prec);
GEN     pasc(long n);
GEN     perf(GEN a);
GEN     polrecip(GEN x);
GEN     polred(GEN x, long prec);
GEN     polred0(GEN x, long flag, GEN p, long prec);
GEN     polred2(GEN x, long prec);
GEN     polredabs(GEN x, long prec);
GEN     polredabs2(GEN x, long prec);
GEN     polredabsall(GEN x, long flun, long prec);
GEN     polredabsfast(GEN x, long prec);
GEN     polredabsnored(GEN x, long prec);
GEN     polsym(GEN x, long n);
GEN     reorder(GEN x);
GEN     setrand(long seed);
GEN     smallpolred(GEN x, long prec);
GEN     smallpolred2(GEN x, long prec);
GEN     sort(GEN x);
GEN     tayl(GEN x, long v, long precdl);
GEN     tchebi(long n, long v);
GEN     vecsort(GEN x, GEN k);
GEN     vecsort0(GEN x, GEN k);

/* bibli2.c */

GEN     dirdiv(GEN x, GEN y);
GEN     direuler(entree *ep, GEN a, GEN b, char *ch);
GEN     dirmul(GEN x, GEN y);
GEN     dirzetak(GEN nf, GEN b);
GEN     divsomme(entree *ep, GEN num, char *ch);
GEN     divsum0(GEN num,entree *ep, char *ch);
void    fordiv(entree *ep, GEN a, char *ch);
void    fordiv0(GEN a, entree *ep, char *ch);
void    forpari(entree *ep, GEN a, GEN b, char *ch);
void    forprime(entree *ep, GEN a, GEN b, char *ch);
void    forstep(entree *ep, GEN a, GEN b, GEN s, char *ch);
void    forvec(entree *ep, GEN x, char *ch);
GEN     genrand(void);
GEN     getheap(void);
GEN     getrand(void);
GEN     getstack(void);
GEN     gettime(void);
GEN     gtoset(GEN x);
long    isvecset(GEN x);
GEN     matrice(entree *ep1, entree *ep2, GEN nlig, GEN ncol, char *ch);
GEN     matrice0(GEN nlig, GEN ncol,entree *ep1, entree *ep2, char *ch);
long    mymyrand();
GEN     permute(long n, GEN x);
GEN     permuteInv(GEN x);
GEN     polint(GEN xa, GEN ya, GEN x, GEN *dy);
GEN     polymodrecip(GEN x);
GEN     polzag(long n, long m);
GEN     polzagreel(long n, long m, long prec);
GEN     prodeuler(entree *ep, GEN a, GEN b, char *ch, long prec);
GEN     prodinf(entree *ep, GEN a, char *ch, long prec);
GEN     prodinf0(entree *ep, GEN a, char *ch, long flag, long prec);
GEN     prodinf1(entree *ep, GEN a, char *ch, long prec);
GEN     produit(entree *ep, GEN a, GEN b, char *ch, GEN x);
GEN     qromb(entree *ep, GEN a, GEN b, char *ch, long prec);
GEN     qromi(entree *ep, GEN a, GEN b, char *ch, long prec);
GEN     qromo(entree *ep, GEN a, GEN b, char *ch, long prec);
GEN     romb0(entree *ep, GEN a, GEN b, char *ch,long flag,long prec);
GEN     rombint(entree *ep, GEN a, GEN b, char *ch, long prec);
GEN     setintersect(GEN x, GEN y);
GEN     setminus(GEN x, GEN y);
long    setsearch(GEN x, GEN y);
GEN     setunion(GEN x, GEN y);
GEN     somme(entree *ep, GEN a, GEN b, char *ch, GEN x);
GEN     sumalt(entree *ep, GEN a, char *ch, long prec);
GEN     sumalt0(entree *ep, GEN a, char *ch,long flag, long prec);
GEN     sumalt2(entree *ep, GEN a, char *ch, long prec);
GEN     suminf(entree *ep, GEN a, char *ch, long prec);
GEN     sumpos(entree *ep, GEN a, char *ch, long prec);
GEN     sumpos0(entree *ep, GEN a, char *ch, long flag,long prec);
GEN     sumpos2(entree *ep, GEN a, char *ch, long prec);
GEN     vecteur(entree *ep, GEN nmax, char *ch);
GEN     vector0(GEN nmax, entree *ep, char *ch);
GEN     vvecteur(entree *ep, GEN nmax, char *ch);
GEN     vvector0(GEN nmax, entree *ep, char *ch);
GEN     zbrent(entree *ep, GEN a, GEN b, char *ch, long prec);

/* buch1.c et buch2.c */

#define buchgen(P,gcbach,gcbach2,prec) (\
  buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,0,prec)\
)
#define buchgenfu(P,gcbach,gcbach2,prec) (\
  buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,2,prec)\
)
#define buchinit(P,gcbach,gcbach2,prec) (\
  buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,-1,prec)\
)
#define buchinitfu(P,gcbach,gcbach2,prec) (\
  buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,-2,prec)\
)

GEN     bnfconductor(GEN arg0,GEN arg1,GEN arg2,long prec);
GEN     bnfconductorofchar(GEN bnr,GEN chi,long prec);
GEN 	bnfinit0(GEN P,long flag,GEN data,long prec);
GEN     buchall(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, GEN gborne, long nbrelpid, long minsfb, long flun, long prec);
GEN     buchfu(GEN bignf);
GEN     buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO);
GEN     buchnarrow(GEN bignf);
GEN     buchray(GEN bignf,GEN ideal,long prec);
GEN     buchray0(GEN bignf, GEN ideal, long flag, long prec);
GEN     buchrayinit(GEN bignf,GEN ideal,long prec);
GEN     buchrayinitgen(GEN bignf,GEN ideal,long prec);
GEN     buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec);
long    certifybuchall(GEN bnf);
GEN 	classgroup0(GEN P,long flag,GEN data,long prec);
GEN 	classgrouponly(GEN P,GEN data,long prec);
GEN     conductor(GEN bnr,GEN subgroup,long prec);
GEN     decodefactor(GEN fa);
GEN     decodemodule(GEN nf, GEN fa);
GEN     discray0(GEN arg0, GEN arg1, GEN arg2, long flag,long prec);
GEN     discrayabs(GEN bnr,GEN subgroup,long prec);
GEN     discrayabscond(GEN bnr,GEN subgroup,long prec);
GEN     discrayabslist(GEN bnf,GEN listes);
GEN     discrayabslistarch(GEN bnf, GEN arch, long bound);
GEN     discrayabslistlong(GEN bnf, long bound);
GEN     discraylist0(GEN bnf,GEN borne, GEN arch, long all);
GEN     discrayrel(GEN bnr,GEN subgroup,long prec);
GEN     discrayrelcond(GEN bnr,GEN subgroup,long prec);
GEN     initbuchray(GEN bignf,GEN ideal,long flag, long prec);
GEN     isprincipal(GEN bignf, GEN x);
GEN     isprincipalall(GEN bignf, GEN x,long flall);
GEN     isprincipalforce(GEN bignf,GEN x);
GEN     isprincipalgen(GEN bignf, GEN x);
GEN     isprincipalgenforce(GEN bignf,GEN x);
GEN     isprincipalray(GEN bignf, GEN x);
GEN     isprincipalrayall(GEN bignf, GEN x,long flall);
GEN     isprincipalraygen(GEN bignf, GEN x);
GEN     isunit(GEN bignf, GEN x);
GEN     makebigbnf(GEN sbnf,long prec);
GEN     quadclassunit0(GEN x, long flag,GEN data, long prec);
GEN     rayclassno(GEN bignf,GEN ideal);
GEN     rayclassnolist(GEN bnf,GEN listes);
GEN 	regulator(GEN P,GEN data,long prec);
GEN     signunit(GEN bignf);
GEN     smallbuchinit(GEN pol,GEN gcbach,GEN gcbach2,GEN gRELSUP,GEN gborne,long nbrelpid,long minsfb,long prec);

/* buch4.c */

GEN     bnfisnorm(GEN bnf,GEN ext,GEN x,long flag);
GEN     bnfissunit(GEN bnf,GEN suni,GEN x);
GEN     bnfsunit(GEN bnf,GEN s);
long    nfhilbert(GEN bnf,GEN a,GEN b);
long    nfhilbert0(GEN bnf,GEN a,GEN b,GEN p);
long    nfhilbertp(GEN bnf,GEN a,GEN b,GEN p);
long    qpsoluble(GEN pol,GEN p);
long    qpsolublenf(GEN bnf,GEN pol,GEN p);
long    zpsoluble(GEN pol,GEN p);
long    zpsolublenf(GEN bnf,GEN pol,GEN p);

/* elliptic.c */

GEN     addell(GEN e, GEN z1, GEN z2);
GEN     akell(GEN e, GEN n);
GEN     anell(GEN e, long n);
GEN     apell(GEN e, GEN p);
GEN     apell2(GEN e, GEN p);
GEN     bilhell(GEN e, GEN z1, GEN z2, long prec);
GEN     coordch(GEN e, GEN ch);
GEN     ellap0(GEN e, GEN p, long flag);
GEN     ellheight0(GEN e, GEN a, long flag,long prec);
GEN     ellinit0(GEN x,long flag,long prec);
GEN     ghell(GEN e, GEN a, long prec);
GEN     ghell2(GEN e, GEN a, long prec);
GEN     globalreduction(GEN e1);
GEN     initell(GEN x, long prec);
GEN     localreduction(GEN e, GEN p1);
GEN     lseriesell(GEN e, GEN s, GEN N, GEN A, long prec);
GEN     mathell(GEN e, GEN x, long prec);
int     oncurve(GEN e, GEN z);
GEN     ordell(GEN e, GEN x, long prec);
GEN     orderell(GEN e, GEN p);
GEN     pointch(GEN x, GEN ch);
GEN     pointell(GEN e, GEN z, long prec);
GEN     powell(GEN e, GEN z, GEN n);
GEN     smallinitell(GEN x);
GEN     subell(GEN e, GEN z1, GEN z2);
GEN     taniyama(GEN e);
GEN     torsell(GEN e);
GEN     weipell(GEN e);
GEN     zell(GEN e, GEN z, long prec);

/* es.c */

char*   GENtostring(GEN x);
void    brute(GEN g, char format, long dec);
void    bruteall(GEN g, char format, long dec, long flbl);
void    bruterr(GEN x,char format,long dec);
void    ecrire(GEN x, char format, long dec, long chmp);
void    etatpile(unsigned int n);
char*   expand_tilde(char *s, char *name);
void    filtre(char *s);
void    flusherr();
void    fprintferr(char* pat, ...);
char*   gitoascii(GEN g, char *buf);
void    matbrute(GEN g, char format, long dec);
void    maxfiltre(char *s);
void    outbeauterr(GEN x);
void    outerr(GEN x);
void    pariputc(char c);
void    pariputs(char *s);
void    sor(GEN g, char fo, long dd, long chmp);
long    switchin(FILE *file);
long    switchin_byname(char *name);
void    switchout(char *name);
void    texe(GEN g, char format, long dec);
FILE *  try_pipe(char *cmd);
void    voir(GEN x, long nb);

/* gen1.c */

GEN     gadd(GEN x, GEN y);
GEN     gdiv(GEN x, GEN y);
GEN     gmul(GEN x, GEN y);
GEN     gsub(GEN x, GEN y);

/* gen2.c gen3.c */

long    opgs2(int (*f) (GEN, GEN), GEN y, long s);
void    gop0z(GEN (*f) (void), GEN x);
void    gop1z(GEN (*f) (GEN), GEN x, GEN y);
void    gop2z(GEN (*f) (GEN, GEN), GEN x, GEN y, GEN z);
void    gop3z(GEN (*f) (GEN, GEN, GEN), GEN x, GEN y, GEN z, GEN t);
void    gopgs2z(GEN (*f) (GEN, GEN), GEN y, long s, GEN z);
void    gops1z(GEN (*f) (long), long s, GEN y);
GEN     gopsg2(GEN (*f) (GEN, GEN), long s, GEN y);
GEN     gopgs2(GEN (*f) (GEN, GEN), GEN y, long s);
void    gops2gsz(GEN (*f) (GEN, long), GEN x, long s, GEN z);
void    gops2sgz(GEN (*f) (long, GEN), long s, GEN y, GEN z);
void    gops2ssz(GEN (*f) (long, long), long s, long y, GEN z);
void    gopsg2z(GEN (*f) (GEN, GEN), long s, GEN y, GEN z);

GEN     Mod0(GEN x, GEN y,long flag);
GEN     centerlift(GEN x);

GEN     cgetp(GEN x);
GEN     co8(GEN x, long l);
GEN     compo(GEN x, long n);
GEN     cvtop(GEN x, GEN p, long l);
GEN     dbltor(double x);
long    degree(GEN x);
GEN     denom(GEN x);
GEN     deriv(GEN x, long v);
GEN     forcecopy(GEN x);
GEN     gabs(GEN x, long prec);
void    gaffect(GEN x, GEN y);
void    gaffsg(long s, GEN x);
GEN     gand(GEN x, GEN y);
GEN     gceil(GEN x);
GEN     gclone(GEN x);
int     gcmp(GEN x, GEN y);
int     gcmp0(GEN x);
int     gcmp1(GEN x);
int     gcmp_1(GEN x);
GEN     gcopy(GEN x);
GEN     gcvtoi(GEN x, long *e);
GEN     gdivent(GEN x, GEN y);
GEN     gdiventres(GEN x, GEN y);
GEN     gdivgs(GEN x, long s);
GEN     gdivmod(GEN x, GEN y, GEN *pr);
GEN     gdivround(GEN x, GEN y);
int     gegal(GEN x, GEN y);
GEN     geq(GEN x, GEN y);
GEN     geval(GEN x);
long    gexpo(GEN x);
GEN     gfloor(GEN x);
GEN     gfrac(GEN x);
GEN     gge(GEN x, GEN y);
GEN     ggprecision(GEN x);
GEN     ggt(GEN x, GEN y);
long    ggval(GEN x, GEN p);
GEN     ginv(GEN x);
GEN     gle(GEN x, GEN y);
GEN     glength(GEN x);
GEN     glt(GEN x, GEN y);
GEN     gmax(GEN x, GEN y);
GEN     gmin(GEN x, GEN y);
GEN     gmod(GEN x, GEN y);
GEN     gmodulcp(GEN x, GEN y);
GEN     gmodulo(GEN x, GEN y);
GEN     gmul2n(GEN x, long n);
GEN     gmulsg(long s, GEN y);
GEN     gne(GEN x, GEN y);
GEN     gneg(GEN x);
GEN     gnot(GEN x);
GEN     gor(GEN x, GEN y);
GEN     gpolvar(GEN y);
int     gprecision(GEN x);
GEN     gpui(GEN x, GEN n, long prec);
GEN     gpuigs(GEN x, long n);
GEN     greffe(GEN x, long l);
GEN     grndtoi(GEN x, long *e);
GEN     ground(GEN x);
GEN     gshift(GEN x, long n);
int     gsigne(GEN x);
long    gsize(GEN x);
GEN     gsqr(GEN x);
GEN     gsubst(GEN x, long v, GEN y);
double  gtodouble(GEN x);
long    gtolong(GEN x);
GEN     gtopoly(GEN x, long v);
GEN     gtopolyrev(GEN x, long v);
GEN     gtoser(GEN x, long v);
GEN     gtovec(GEN x);
GEN     gtrunc(GEN x);
int     gvar(GEN x);
int     gvar2(GEN x);
GEN     integ(GEN x, long v);
int     iscomplex(GEN x);
int     isexactzero(GEN g);
int     isinexactreal(GEN x);
int     ismonome(GEN x);
GEN     karamul(GEN x, GEN y, long k);
int     lexcmp(GEN x, GEN y);
GEN     lift(GEN x);
GEN     matsize(GEN x);
GEN     mpkaramul(GEN x, GEN y, long k);
void    normalize(GEN *px);
void    normalizepol(GEN *px);
GEN     numer(GEN x);
long    padicprec(GEN x, GEN p);
int     polegal(GEN x, GEN y);
int     precision(GEN x);
GEN     precision0(GEN x,long n);
long    pvaluation(GEN x, GEN p, GEN *py);
GEN     recip(GEN x);
GEN     round0(GEN x, long flag);
long    rounderror(GEN x);
double  rtodbl(GEN x);
GEN     simplify(GEN x);
long    taille(GEN x);
long    taille2(GEN x);
GEN     truecoeff(GEN x, long n);
GEN     trunc0(GEN x, long flag);
int     vecegal(GEN x, GEN y);
GEN     vecmax(GEN x);
GEN     vecmin(GEN x);

/* init.c */

#ifdef MALLOC_PROCS
  extern Malloc_procs malloc_procs;
#endif

GEN     allocatemem(uLong newsize);
void    allocatemoremem(uLong newsize);
void    changevalue(entree *ep, GEN val);
void    freeall(void);
GEN     gerepile(long ltop, long lbot, GEN q);
void    gerepilemany(long av, GEN* g[], long n);
void    gerepilemanysp(long av, long tetpil, GEN* g[], long n);
void    gerepilemanyvec(long av, long tetpil, long *g, long n);
GEN     gerepileupto(long av, GEN q);
void    pari_init(long parisize, long maxprime);
entree* install(void *f, char *name, int valence);
void    killall(void);
void    killbloc(GEN x);
void    killvalue(entree *ep);
GEN     newbloc(long n);
void    newvalue(entree *ep, GEN val);
void    recover(long listloc);
long    timer(void);
long    timer2(void);

BEGINEXTERN
char*   xmalloc(size_t bytes);
char*   xrealloc(void *pointer,size_t newsize,size_t oldsize);
VOLATILE void err(long numerr, ...);
ENDEXTERN

/* nffactor.c */

GEN     nffactor(GEN nf,GEN x, long klim);
GEN     nffactormod(GEN nf,GEN pol,GEN pr);
GEN     nfroots(GEN nf,GEN pol);
GEN     rnfcharpoly(GEN nf,GEN T,GEN alpha,int n);
GEN     rnfdedekind(GEN nf,GEN T,GEN pr);

/* polarit.c */

GEN     apprgen(GEN f, GEN a);
GEN     apprgen9(GEN f, GEN a);
GEN     bezoutpol(GEN a, GEN b, GEN *u, GEN *v);
GEN     content(GEN x);
GEN     discsr(GEN x);
GEN     factcantor(GEN x, GEN p);
GEN     factmod(GEN f, GEN p);
GEN     factormod0(GEN f, GEN p,long flag);
GEN     factmod2(GEN f, GEN p);
GEN     factmod9(GEN f, GEN p, GEN a);
GEN     factor(GEN x);
GEN     factor0(GEN x,long flag);
GEN     factorpadic0(GEN f,GEN p,long r,long flag);
GEN     factorpadic2(GEN x, GEN p, long r);
GEN     factorpadic4(GEN x, GEN p, long r);
GEN     factpol(GEN x, long klim, long hint);
GEN     factpol2(GEN x, long klim);
GEN     gbezout(GEN x, GEN y, GEN *u, GEN *v);
GEN     gcd0(GEN x, GEN y,long flag);
GEN     gcvtop(GEN x, GEN p, long r);
GEN     gdeuc(GEN x, GEN y);
int     gdivise(GEN x, GEN y);
GEN     ggcd(GEN x, GEN y);
GEN     ginvmod(GEN x, GEN y);
GEN     gisirreducible(GEN x);
GEN     glcm(GEN x, GEN y);
GEN     gred(GEN x);
void    gredsp(GEN *px);
GEN     gres(GEN x, GEN y);
GEN     newtonpoly(GEN x, GEN p);
GEN     nfincl(GEN a, GEN b);
GEN     nfisincl0(GEN a, GEN b,long flag);
GEN     nfisisom0(GEN a, GEN b,long flag);
GEN     nfiso(GEN a, GEN b);
int     poldivis(GEN x, GEN y, GEN *z);
GEN     poldivres(GEN x, GEN y, GEN *pr);
GEN     poleval(GEN x, GEN y);
GEN     polfnf(GEN a, GEN t);
GEN     polgcd(GEN x, GEN y);
GEN     primpart(GEN x);
GEN     quadgen(GEN x);
GEN     quadpoly(GEN x);
GEN     reduceddiscsmith(GEN pol);
GEN     resultant0(GEN x, GEN y, long flag);
GEN     resultant2(GEN x, GEN y);
GEN     rootmod(GEN f, GEN p);
GEN     rootmod0(GEN f, GEN p,long flag);
GEN     rootmod2(GEN f, GEN p);
GEN     rootpadic(GEN f, GEN p, long r);
GEN     rootpadicfast(GEN f, GEN p, long r, long flall);
GEN     roots2(GEN pol,long PREC);
GEN     rootsold(GEN x, long l);
GEN     simplefactmod(GEN f, GEN p);
void    split(long m, GEN *t, long d, long p, GEN q);
GEN     squarefree(GEN f);
GEN     srgcd(GEN x, GEN y);
long    sturm(GEN x);
long    sturmpart(GEN x, GEN a, GEN b);
GEN     subres(GEN x, GEN y);
GEN     subresext(GEN x, GEN y, GEN *U, GEN *V);
GEN     sylvestermatrix(GEN x,GEN y);
GEN     vecbezout(GEN x, GEN y);
GEN     vecbezoutres(GEN x, GEN y);

/* rootpol.c */

int     isrealappr(GEN x, long l);
GEN     modulargcd(GEN a,GEN b);
GEN     mulii(GEN x, GEN y);
GEN     sqri(GEN x);
GEN     roots(GEN x,long l);
GEN     roots0(GEN x,long flag,long l);

/* subfields.c */

GEN     Fpn(GEN p,long n);
GEN     subfields(GEN nf,GEN d);
GEN     subfields0(GEN nf,GEN d);
GEN     conjugates(GEN pol);

/* thue.c */

GEN     thue(GEN thueres, GEN rhs, GEN ne);
GEN 	thueinit(GEN poly);

/* trans.c */

GEN     agm(GEN x, GEN y, long prec);
GEN     bernreal(long n, long prec);
GEN     bernvec(long nomb);
void    consteuler(long prec);
void    constpi(long prec);
GEN     dilog(GEN x, long prec);
GEN     eint1(GEN x, long prec);
GEN     eta(GEN x, long prec);
GEN     gach(GEN x, long prec);
void    gachz(GEN x, GEN y);
GEN     gacos(GEN x, long prec);
void    gacosz(GEN x, GEN y);
GEN     garg(GEN x, long prec);
GEN     gash(GEN x, long prec);
void    gashz(GEN x, GEN y);
GEN     gasin(GEN x, long prec);
void    gasinz(GEN x, GEN y);
GEN     gatan(GEN x, long prec);
void    gatanz(GEN x, GEN y);
GEN     gath(GEN x, long prec);
void    gathz(GEN x, GEN y);
GEN     gch(GEN x, long prec);
void    gchz(GEN x, GEN y);
GEN     gcos(GEN x, long prec);
void    gcosz(GEN x, GEN y);
GEN     gerfc(GEN x, long prec);
GEN     gexp(GEN x, long prec);
void    gexpz(GEN x, GEN y);
GEN     ggamd(GEN x, long prec);
void    ggamdz(GEN x, GEN y);
GEN     ggamma(GEN x, long prec);
void    ggammaz(GEN x, GEN y);
GEN     gimag(GEN x);
GEN     glngamma(GEN x, long prec);
void    glngammaz(GEN x, GEN y);
GEN     glog(GEN x, long prec);
GEN     glogagm(GEN x, long prec);
void    glogz(GEN x, GEN y);
GEN     gpolylog(long m, GEN x, long prec);
void    gpolylogz(long m, GEN x, GEN y);
GEN     gpsi(GEN x, long prec);
void    gpsiz(GEN x, GEN y);
GEN     greal(GEN x);
GEN     gsh(GEN x, long prec);
void    gshz(GEN x, GEN y);
GEN     gsin(GEN x, long prec);
void    gsincos(GEN x, GEN *s, GEN *c, long prec);
void    gsinz(GEN x, GEN y);
GEN     gsqrt(GEN x, long prec);
void    gsqrtz(GEN x, GEN y);
GEN     gtan(GEN x, long prec);
void    gtanz(GEN x, GEN y);
GEN     gth(GEN x, long prec);
void    gthz(GEN x, GEN y);
GEN     gzeta(GEN x, long prec);
void    gzetaz(GEN x, GEN y);
GEN     hyperu(GEN a, GEN b, GEN gx, long prec);
GEN     incgam(GEN a, GEN x, long prec);
GEN     incgam0(GEN a, GEN x, GEN z,long prec);
GEN     incgam1(GEN a, GEN x, long prec);
GEN     incgam2(GEN a, GEN x, long prec);
GEN     incgam3(GEN a, GEN x, long prec);
GEN     incgam4(GEN a, GEN x, GEN z, long prec);
GEN     izeta(GEN x, long prec);
GEN     jbesselh(GEN n, GEN z, long prec);
GEN     jell(GEN x, long prec);
GEN     kbessel(GEN nu, GEN gx, long prec);
GEN     kbessel0(GEN nu, GEN gx, long flag,long prec);
GEN     kbessel2(GEN nu, GEN x, long prec);
GEN     log0(GEN x,long flag, long prec);
GEN     logagm(GEN q);
void    mpbern(long nomb, long prec);
GEN     mpeuler(long prec);
GEN     mpexp(GEN x);
GEN     mpexp1(GEN x);
void    mpgamdz(long s, GEN y);
GEN     mplog(GEN x);
GEN     mppi(long prec);
GEN     mpsqrt(GEN x);
GEN     palog(GEN x);
GEN     polylog(long m, GEN x, long prec);
GEN     polylog0(long m, GEN x, long flag, long prec);
GEN     polylogd(long m, GEN x, long prec);
GEN     polylogdold(long m, GEN x, long prec);
GEN     polylogp(long m, GEN x, long prec);
GEN     teich(GEN x);
GEN     theta(GEN q, GEN z, long prec);
GEN     thetanullk(GEN q, long k, long prec);
GEN     transc(GEN (*f) (GEN, long), GEN x, long prec);
GEN     wf(GEN x, long prec);
GEN     weber0(GEN x, long flag,long prec);
GEN     wf2(GEN x, long prec);

/* version.c */

GEN     gerepilc(GEN l, GEN p, GEN q);

BEGINEXTERN
int     bfffo(uLong);
long    divll(uLong, uLong); /* comment ?? */
ENDEXTERN

#if !defined(__cplusplus) && !defined(FASTASM) 
long    addll(uLong, uLong);
long    addllx(uLong, uLong);
long    addmul(uLong, uLong);
long    mulll(uLong, uLong);
long    shiftl(uLong, uLong);
long    shiftlr(uLong, uLong);
long    subll(uLong, uLong);
long    subllx(uLong, uLong);
#endif
