/*******************************************************************/
/*******************************************************************/
/*                                                                 */
/*              PROGRAMME D'INITIALISATION DU SYSTEME              */
/*                    ET TRAITEMENT DES ERREURS                    */
/*                                                                 */
/*                       copyright Babe Cool                       */
/*                                                                 */
/*******************************************************************/
/*******************************************************************/
/* $Id: init.c,v 1.914.7.1 1997/01/19 16:18:09 karim Exp karim $ */

#include <string.h>
#include "genpari.h"
#include "gp.h"
						 /* let SL = sizeof(long) */
const double K = 9.632959862*(BYTES_IN_LONG/4);  /* SL*log(2)/log(10)     */
const double K1 = 0.103810253/(BYTES_IN_LONG/4); /* log(10)/(SL*log(2))   */
const double K2 = 1.1239968;                     /* 1/(1-(log(2)/(2*pi))) */
const double K4 = 17.079468445347/BITS_IN_LONG;  /* 2*e*pi/SL             */
const double LOG2 = 0.69314718055994531;         /* log(2)                */
const double L2SL10 = 0.301029995663981;         /* log(2)/log(10)        */
const double C1 = 0.9189385332;                  /* log(2*pi)/2           */
const double C2 = 22.18070978*(BYTES_IN_LONG/4); /* SL*log(2)             */
const double C3 = 0.0216950598/(BYTES_IN_LONG/4);/* log((1+sqrt(5))/2)/C2 */

#ifndef  PI
  const double PI = 3.141592653589;
#endif

#ifdef LONG_IS_64BIT
  const double C31 = 9223372036854775808.0;    /* 2^63   */
  const long VERYBIGINT = 9223372036854775807; /* 2^63-1 */
  const long EXP220 = 1099511627776;           /* 2^40   */
  const long BIGINT = 2147483647;              /* 2^31-1 */
#else
  const double C31 = 2147483648.0;    /* 2^31   */
  const long VERYBIGINT = 2147483647; /* 2^31-1 */
  const long EXP220 = 1048576;        /* 2^20   */
  const long BIGINT = 32767;          /* 2^15-1 */
#endif

/*      Variables statiques communes :         */
FILE    *outfile, *errfile, *logfile, *infile;
GEN     *polun, *polx;
GEN     gnil, gzero, gun, gdeux, ghalf, polvar, gi, RAVYZARC;
GEN     gpi=NULL, geuler=NULL, bernzone=NULL;
GEN     primetab; /* private primetable */
byteptr diffptr;
char    current_logfile[128], current_psfile[128], *current_function;

int     functions_tblsz = 135; /* size of functions_hash          */
entree  **varentries;

jmp_buf environnement;
long    *ordvar;
long    DEBUGLEVEL, compatible;
long    try_to_recover=1, avloc=0, pari_randseed;
long    prec,precdl,defaultpadicprecision;
long    paribuffsize=30000;
uLong   init_opts = INIT_JMPm | INIT_SIGm;
uLong   top, bot, avma;

void *foreignHandler; 	              /* Handler for foreign commands.   */
char foreignExprSwitch = 3; 	      /* Just some unprobable char.      */
GEN  (*foreignExprHandler)(char*);    /* Handler for foreign expressions.*/
long (*foreignAutoload)(char*, long); /* Autoloader                      */
void (*foreignFuncFree)(entree *);    /* How to free external entree.    */

GEN  (*gp_history_fun)(long, long, char *, char *);

/*********************************************************************/
/*                                                                   */
/*                     INITIALISATION DU SYSTEME                     */
/*                                                                   */
/*********************************************************************/

static int check_gerepile=1;
static long next_bloc;
static GEN cur_bloc=NULL; /* current bloc in bloc list */

static void
pari_sighandler(int sig)
{
  char *msg;
  switch(sig)
  {
    case SIGINT:
      if (infile != stdin)
      {
	int file_depth = switchin(NULL);
	if (file_depth == -1) { freeall(); exit(0); }
      }
      msg="user interrupt";
      break;

    case SIGSEGV:
      msg="segmentation fault: bug in PARI or calling program";
      break;

#ifdef SIGBUS
    case SIGBUS:
      msg="bus error: bug in PARI or calling program";
      break;
#endif
  }
  signal(sig,pari_sighandler);
  err(talker,msg);
}

/* Initialize hashtable */
static void
init_hashtable(entree **table, int tblsz)
{
  long n;
  for (n = 0; n < tblsz; n++) table[n] = NULL;
}

/* Fill hashtable */
static void
fill_hashtable(entree **table, int tblsz, entree *fun, char **helpmessage)
{
  long n,v;

  for (v = 0; fun[v].name; v++)
  {
    fun[v].help = helpmessage[v];
    fun[v].valence |= EpSTATIC;
    n=hashvalue(&fun[v].name,tblsz,0);
    fun[v].next = table[n];
    table[n] = fun + v;
  }
}

void
init_defaults(int force)
{
  static int done=0;

  if (done && !force) return;
  done = 1;

#ifdef LONG_IS_64BIT
  prec=4;
#else
  prec=5;
#endif

  precdl = defaultpadicprecision = 16;
  compatible = NONE;
  DEBUGLEVEL = 0;
  pari_randseed = 1;

  strcpy(current_psfile,"pari.ps");
  strcpy(current_logfile,"pari.log");

  infile = stdin; outfile = stdout;
  errfile = stderr; logfile = NULL;
  next_bloc=0;
}

/* does elt belong to list, after position start (excluded) ? */
static int
list_isin(void **list, void *elt, int start)
{
  long indelt=0;

  if (list)
  {
    while (*list)
    {
      if (indelt>start && *list==elt) return indelt;
      list++; indelt++;
    }
  }
  return -1;
}

static void
list_prepend(void ***listptr, void *elt)
{
  void **list=*listptr;
  long nbelt=0;

  if (list)
    while (list[nbelt]) nbelt++;
  list = (void **) xmalloc(sizeof(void *)*(nbelt+2));
  list[0]=elt;
  if (nbelt)
  {
    memcpy(list+1,*listptr,nbelt*sizeof(void *));
    free(*listptr);
  }
  list[nbelt+1]=NULL; *listptr=list;
}

/* Load modlist in hashtable hash. If force == 0, do not load twice the 
 * same list in the same hashtable, which would only destroy user variables.
 * As it stands keep a complete history (instead of most recent changes).
 */
int
gp_init_entrees(module *modlist, entree **hash, int force)
{
  static void **oldmodlist=NULL, **oldhash=NULL;

  if (!force) 
  {
    const long indhash = list_isin(oldhash,(void *)hash,-1);
    if (indhash != -1 && oldmodlist[indhash]==modlist) return 0;
  }
  /* record the new pair (hash,modlist) */
  list_prepend(&oldmodlist,(void *)modlist);
  list_prepend(&oldhash,(void *)hash);

  init_hashtable(hash,functions_tblsz);
  while (modlist && modlist->func)
  {
    fill_hashtable(hash,functions_tblsz, modlist->func, modlist->help);
    modlist++;
  }
  if (hash != functions_hash) return 0;
  manage_var(2,NULL); /* init nvar */
  reorder(NULL); lisseq("x"); return 1;
}

module *pari_modules    = NULL;
module *pari_oldmodules = NULL;
entree **functions_hash = NULL;
entree **funct_old_hash = NULL;

/* add to modlist the functions in func, with helpmsg help */
void
pari_addfunctions(module **modlist_p, entree *func, char **help)
{
  module *modlist = *modlist_p;
  int nbmodules = 0;

  while (modlist && modlist->func) { nbmodules++; modlist++; }
  modlist = *modlist_p;
  *modlist_p = (module*) xmalloc(sizeof(module)*(nbmodules+2));
  if (nbmodules)
  {
    memcpy(1+ *modlist_p, modlist, sizeof(module)*nbmodules);
    free(modlist);
  }
  modlist = *modlist_p;
  modlist->func = func;
  modlist->help = help;

  modlist += nbmodules+1;
  modlist->func = NULL;
  modlist->help = NULL;
}

/* BYTES_IN_LONG*ceil(a/BYTES_IN_LONG) */
#define fix_size(a) ((a)+BYTES_IN_LONG - ((((a)-1) & (BYTES_IN_LONG-1)) + 1))

/* initialise les donnees de la bibliotheque PARI. Peut tre prcde d'un
 * appel  pari_addfunctions si on ajoute d'autres fonctions au pool de base.
 */
void
pari_init(long parisize, long maxprime)
{
  long n,dummy;
  GEN p;

  init_defaults(0);
  if (INIT_JMP && setjmp(environnement))
  {
    fprintferr("\n  ***   Error in the PARI system. End of program.\n");
    exit(1);
  }

  if (INIT_SIG)
  {
#ifdef SIGBUS
    signal(SIGBUS,pari_sighandler);
#endif
    signal(SIGINT,pari_sighandler);
    signal(SIGSEGV,pari_sighandler);
  }

  parisize = fix_size(parisize);
  diffptr = initprimes(maxprime);
#if __MWERKS__
  {
    OSErr resultCode;
    Handle newHand = MFTempNewHandle(parisize,&resultCode);

    if (!newHand) err(memer);
    HLock(newHand);
    bot=(long)*newHand;
  }
#else
  bot=(long) xmalloc(parisize);
#endif
  top = avma = bot+parisize;

  varentries = (entree **) xmalloc(sizeof(entree*)*(MAXVARN+1));
  ordvar = (long *) xmalloc(sizeof(long)*(MAXVARN+1));
  polun = (GEN *) xmalloc(sizeof(GEN)*(MAXVARN+1));
  polx = (GEN *) xmalloc(sizeof(GEN)*(MAXVARN+1));

  gnil  = cgeti(2); gnil[1]=2;  setpere(gnil,MAXPERE);
  gzero = cgeti(2); gzero[1]=2; setpere(gzero, MAXPERE);
  
  gun = stoi(1);   setpere(gun, MAXPERE);
  gdeux = stoi(2); setpere(gdeux, MAXPERE);

  ghalf = cgetg(3,t_FRAC); ghalf[1]=un; ghalf[2]=deux;
  gi = cgetg(3,t_COMPLEX); gi[1] = zero; gi[2] = un;
  setpere(ghalf, MAXPERE); setpere(gi, MAXPERE);

  p=cgetg(4,t_POL); polx[MAXVARN]=p; p[2]=zero; p[3]=un;
  p[1] = evalsigne(1) | evalvarn(MAXVARN) | evallgef(4);
  p=cgetg(3,t_POL); polun[MAXVARN]=p; p[2]=un;
  p[1] = evalsigne(1) | evalvarn(MAXVARN) | evallgef(3);
  for (n=0; n <= MAXVARN; n++) ordvar[n] = n;

  polvar = cgetg(MAXVAR + 1,t_VEC); setlg(polvar,1); setpere(polvar, MAXPERE);
  dummy = evaltyp(t_VEC) | evalpere(MAXPERE) | evallg(1);
  for (n=1; n<=MAXVAR; n++) polvar[n]=dummy;

  primetab = cgetg(NUMPRTBELT+2,t_VEC);
  for (n = 1; n <= NUMPRTBELT+1; n++) primetab[n]=un;

  pari_addfunctions(&pari_modules, functions_basic,helpmessages_basic);
  functions_hash = (entree **) xmalloc(sizeof(entree*)*functions_tblsz);
  gp_init_entrees(pari_modules, functions_hash, 1);

  /* Les trois lignes suivantes sont pour la compatibilit. */
  pari_addfunctions(&pari_oldmodules, oldfonctions,oldhelpmessage);
  funct_old_hash = (entree **) xmalloc(sizeof(entree*)*functions_tblsz);
  gp_init_entrees(pari_oldmodules, funct_old_hash, 1);

  gp_history_fun = NULL; avloc=avma;
}

void
freeall()
{
  long i;
  entree *ep,*ep1;

  free((void *)polx); free((void *)polun); free((void *)ordvar);
  if (gp_history_fun)
    gp_history_fun(0,-2,NULL,NULL);
  while (delete_var()) /* empty */;

  for (i = 0; i < functions_tblsz; i++)
    for (ep = functions_hash[i]; ep; ep = ep1)
    {
      ep1 = ep->next;
      freeep(ep);
    }
  free((void *)functions_hash); free((void *)varentries);
  free((void *)bot); free((void *)diffptr);
  while (cur_bloc) killbloc(cur_bloc);
}

GEN
getheap()
{
  long m=0,l=0;
  GEN x = cur_bloc;

  for (; x; x = (GEN)x[-3]) { m++; l += taille(x)+4; }
  x=cgetg(3,t_VEC); x[1]=lstoi(m); x[2]=lstoi(l);
  return x;
}

long
marklist(void)
{
  return next_bloc;
}

/*
 * Return x, where:
 * x[-4]: adress of next bloc (set to NULL).
 * x[-3]: adress of preceding bloc.
 * x[-2]: number of allocated blocs.
 * x[-1]: original value of x (GEN set to NULL)
 * x[0..n-1]: malloc-ed memory.
 */
GEN
newbloc(long n)
{
  long *x = (long *) xmalloc((n << TWOPOTBYTES_IN_LONG) + 4*sizeof(long));
  *x++ = 0;
  *x++ = (long)cur_bloc;
  *x++ = next_bloc++;
  *x++ = 0;
  if (cur_bloc) cur_bloc[-4] = (long)x;
  return cur_bloc = x;
}

void
killbloc(GEN x)
{
  if (!x || isonstack(x)) return;
  if (x[-4]) ((GEN)x[-4])[-3] = x[-3]; else cur_bloc = (GEN)x[-3];
  if (x[-3]) ((GEN)x[-3])[-4] = x[-4];
  free((void *)(x-4));
}

void
newvalue(entree *ep, GEN val)
{
  GEN y = gclone(val);

  y[-1] = (long) ep->value;
  ep->value = (void *)y;
  ep->code = NULL;
}

void
changevalue(entree *ep, GEN val)
{
  GEN y = gclone(val), x = (GEN)ep->value;

  ep->value = (void *)y;
  if ((long)x - (long)ep == sizeof(entree))
  {
    y[-1] = (long)x;
    return;
  }
  y[-1] = x[-1];
  killbloc(x);
}

void
killvalue(entree *ep)
{
  GEN x = (GEN)ep->value;

  if ((long)x - (long)ep == sizeof(entree)) return;
  ep->value = (void *)x[-1];
  killbloc(x);
}

/*******************************************************************/
/*                                                                 */
/*                       TRAITEMENT DES ERREURS                    */
/*                                                                 */
/*******************************************************************/

/*
 * Outputs a "beautiful" error message.
 *   msg is errmessage to print.
 *   s points to the offending chars.
 *   entry tells how much we can go back from s[0].
 */
static void
errcontext(char *msg, char *s, char *entry)
{
  int i, len2, past = (s-entry);
  char *t, *dots = "", str[80], str2[21]; ;

  if (past && s[-1] == '\n') /* can only happen in the middle of a string */
  {
    while (past && isspace(s[-1])) { s--; past--; }
  }
  if (past>25) { dots="..."; past=25; }
  sprintf(str, "%s%s", msg, dots);
  pariputs(str); t = str+strlen(str);
  strncpy(t, s - past, past); t[past]=0;
  pariputs(t); t += strlen(t);

  strncpy(str2,s,20); str2[20]=0; /* in case strlen(s) > 19 */
  len2=strlen(str2); str2[len2--]=0;
  if (str2[len2]=='\n')
  {
    /* comes from the timer in gp, i.e. # */
    str2[len2--]=0;
  }
  pariputs(str2); pariputc('\n');

  while (t > str)
  {
    if (*--t == '\n') break;
    pariputc(' ');
  }
  pariputc('^');
  for (i=0; i<len2; i++) pariputc('-');
  pariputc('\n');
}

void
err_recover(long numerr)
{
  if (!pariErr || !pariErr->die) flusherr();
  outfile=stdout; errfile=stderr;
  if (!pariErr || !pariErr->putc) fprintferr("\n");
  if (pariErr && pariErr->die) pariErr->die();
  if (environnement && try_to_recover) longjmp(environnement, numerr);
  exit(1);
}

VOLATILE
void
err(long numerr, ...)
{
  char str[80], *ch1, *ch2;
  va_list poer;

  va_start(poer,numerr);
  if (numerr != memer) { flusherr(); fflush(outfile); }
  else pariErr=NULL; /* Try to avoid loop */

  sprintf(str,"  ***   %s",errmessage[numerr]);
  switch (numerr)
  {
    case arrayer1: case caracer1: case caseer2: case caseer: case expter1:
    case inputer1: case outputer1: case outloger1:
    case killer1: case labeler: case nparamer1: case numvarer:
    case paramer1: case recter1: case referer1: case referer2: case trucer1:
    case varer1: case vectmater1:
      ch1=va_arg(poer,char *); ch2=va_arg(poer,char *);
      errcontext(str,ch1,ch2); break;

    case talker2:
      strcat(str,va_arg(poer, char*));
      ch1=va_arg(poer,char *); ch2=va_arg(poer,char *);
      errcontext(str,ch1,ch2); break;

   case member:
      strcat(str,va_arg(poer, char*)); strcat(str,": ");
      ch1=va_arg(poer,char *); ch2=va_arg(poer,char *);
      errcontext(str,ch1,ch2); break;

/* no context for these */
    case errpile:
      fprintferr(str);
      if (!pariErr || !pariErr->putc) fprintferr("\n");
      allocatemoremem(0); break;

    case flagerr:
      fprintferr(" Invalid flag in %s.",current_function); break;

    case invmoder:
    {
      GEN noninv;
      fprintferr(str);
      ch1=va_arg(poer, char*); noninv=va_arg(poer, GEN);
      fprintferr(": "); bruterr(noninv,'g',-1); break;
    }
    case impl:
      fprintferr(str);
      ch1=va_arg(poer, char*);
      fprintferr(" %s is not yet implemented.",ch1); break;

    case talker:
      fprintferr(str);
      ch1=va_arg(poer, char*);
      fprintferr("%s.",ch1); break;

    default:
      fprintferr(str);
  }
  va_end(poer); err_recover(numerr);
  exit(1); /* not reached */
}

void
recover(long listloc)
{
  long m,n;
  GEN x,y;
  entree *ep;

  for (n = 0; n < functions_tblsz; n++)
    for (ep = functions_hash[n]; ep; )
      if (EpVALENCE(ep) >= 100)
      {
        x = (GEN)ep->value;
        if ((long)x - (long)ep == sizeof(entree))
        {
          if (EpVALENCE(ep) == 200) ep = ep->next;
          else
            if (ep == functions_hash[n])
            {
              functions_hash[n] = ep->next;
              freeep(ep);
              ep = functions_hash[n];
            }
            else
            {
	      entree *ep2 = functions_hash[n];
              while (ep2->next != ep) ep2 = ep2->next;
              ep2->next = ep->next;
              freeep(ep); ep = ep2->next;
            }
          continue;
        }
	m = x[-2];
        if (m < listloc || m >= next_bloc) ep=ep->next;
        else killvalue(ep);
      }
      else ep = ep->next;
  for (x = cur_bloc; x && x[-2] >= listloc; x = y)
  {
    y = (GEN)x[-3];
    if (x != gpi && x != geuler) killbloc(x);
  }
}

/*  Takes an array of pointers to GENs, of length n. Copies all
 *  objects to contiguous locations and cleans up the stack between
 *  ltop and avma.
 */
void
gerepilemany(long ltop, GEN* gptr[], long n)
{
  const long lbot = avma;
  long lbot2, i, dec;

  for (i=0; i<n; i++)
  {
    GEN *g1 = gptr[i];
    long l1 = (long)(*g1);
    if (l1 >= lbot && l1 <= ltop) *g1 = gcopy(*g1); /* Copy if not in heap */
  }
  lbot2 = avma;

  dec = lpile(ltop, lbot, 0) >> TWOPOTBYTES_IN_LONG;

  for (i=0; i<n; i++)
  {
    GEN *g1 = gptr[i];
    long l1 = (long)(*g1);
    if (l1 >= lbot2 && l1 < lbot) *g1 += dec; /* Update addresses */
  }
}

/*  Takes an array of pointers to GENs, of length n.
 *  Cleans up the stack between av and tetpil, updating those GENs.
 */
void
gerepilemanysp(long av, long tetpil, GEN* gptr[], long n)
{
  const long lbot=avma, dec = lpile(av, tetpil, 0) >> TWOPOTBYTES_IN_LONG;
  long i;

  for (i=0; i<n; i++,gptr++)
  {
    GEN *gg = *gptr;
    long l = (long)(*gg);
    if (l < tetpil)
    {
      if (l >= lbot ) *gg += dec; /* Update addresses if in stack */
      else if (l >=av) err(gerper); /* Check if pointer got lost */
    }
  }
}

/*  Takes an array of GENs (casted to longs), of length n. Cleans up the
 *  stack between av and tetpil, and update those GENs.
 */
void
gerepilemanyvec(long av, long tetpil, long *g, long n)
{
  const long lbot = avma, dec = lpile(av, tetpil, 0);
  long i;

  for (i=0; i<n; i++,g++)
  {
    if (*g < tetpil)
    {
      if (*g >= lbot ) *g += dec; /* Update addresses if in stack */
      else if (*g >=av) err(gerper); /* Check if pointer got lost */
    }
  }
}

#ifdef __M68K__
#  define gerepile gerepilec
#endif

GEN
gerepile(long l, long p, GEN q)
{
  long av,dec=l-p;
  GEN ll,l2,l3;

  if (dec==0) return q;
  if (dec<0 && check_gerepile)
    err(talker,"lbot>ltop in gerepile");

#if 0 /* slower... sigh */
  av = avma + dec;
  if (av>=p) /* no overlap */
  {
    memcpy((char *) av, (char *) avma, p-avma);
    ll = (GEN) av;
  }
  else
#endif
  {
    GEN pp;

    for (ll=(GEN)l, pp=(GEN)p; pp > (GEN)avma; ) *--ll= *--pp;
    av = (long) ll;
  }

  while (ll < (GEN)l)
  {
    const long tl=typ(ll);

    switch(tl)
    {
      case t_INT: case t_REAL:
	ll+=lg(ll); continue;

      case t_POL:
	l2=ll+lontyp[tl]; l3=ll+lgef(ll); ll+=lg(ll); break;

      case t_SER:
        if (!signe(ll)) { ll+=lg(ll); continue; } /* fall through */

      default:
	l2=ll+lontyp[tl]; ll+=lg(ll); l3=ll;
    }

    for (; l2<l3; l2++)
    {
      const GEN l1=(GEN)(*l2);

      if ( l1 < (GEN)l && l1 >= (GEN)avma )
      {
	if (l1<(GEN)p) *l2 += dec;
	else
	  if (ll<=(GEN)l && check_gerepile) err(gerper);
      }
    }
  }
  if ( !((long)q) || (q<(GEN)p && q>=(GEN)avma) )
  {
    avma=av; return q+(dec>>TWOPOTBYTES_IN_LONG);
  }
  else { avma=av; return q; }
}

#ifdef __M68K__
#  undef gerepile
#endif

GEN
gerepileupto(long av, GEN q)
{
  /* Beware: (long)(q+i) --> ((long)q)+i*sizof(long) */
  return gerepile(av,(long)(q+lg(q)),q);
}

void
allocatemoremem(uLong newsize)
{
  long declg,v, sizeold = top - bot;
  unsigned long topold = top, avmaold = avloc;

  if (!avloc)
  {
    fprintferr("  *** Can't recover. parisize = %ld was not enough !\n",
               sizeold);
    exit(1);
  }
  if (!newsize)
  {
    newsize = sizeold << 1;
    fprintferr("  *** Warning: doubling stack size; new stack = %ld\n",newsize);
  }
  else
  {
    v = topold - avmaold;
    if (newsize < v)
      err(talker,"required stack memory too small");
    if (newsize < v<<1)
      err(talker,"too dangerous to do that");

    newsize = fix_size(newsize);
  }
  if (newsize<sizeold)
  {
    /* we need the old bot during gerepile */
    v = bot; bot = (long) xmalloc(newsize);
  }
  else
    bot = (long) xrealloc((char *)bot, newsize, sizeold);
  top = bot + newsize;

  check_gerepile=0; avma = avmaold;
#ifndef __M68K__
  declg = (long) gerepile(top,topold,0) >> TWOPOTBYTES_IN_LONG;
#else
  declg = (long) gerepilec(top,topold,0) >> TWOPOTBYTES_IN_LONG;
#endif
  check_gerepile=1; avloc = avma;
  if (newsize < sizeold) free((void *)v);

  gnil+=declg; gzero+=declg; gun+=declg; gdeux+=declg; ghalf+=declg;
  gi+=declg; polvar+=declg; polx[MAXUBYTE]+=declg; polun[MAXUBYTE]+=declg;
  if (gp_history_fun)
    gp_history_fun(declg,-1,(char *)avmaold,(char *)topold);
}

BEGINEXTERN
char*
xmalloc(size_t bytes)
{
  char *tmp;

  if (bytes)
  {
    tmp = (char *) malloc(bytes);
    if (!tmp) err(memer);
    return tmp;
  }
  if (DEBUGLEVEL>6)
  {
    fprintferr("Warning, mallocing NULL object\n");
    flusherr();
  }
  return NULL;
}

#if __MWERKS__
static
void *macrealloc(void *p, size_t newsize, size_t oldsize)
{
  char *q = xmalloc(newsize), *qq = q, *pp = p;
  int l = newsize > oldsize ? oldsize : newsize;

  while (l--) *qq++ = *pp++;
  free(p); return q;
}
#endif

char*
xrealloc(void *pointer, size_t newsize, size_t oldsize)
{
  char *tmp;

  if (!pointer) tmp = (char *) malloc(newsize);
#if __MWERKS__
  else tmp = (char *) macrealloc(pointer,newsize,oldsize);
#else
  else tmp = (char *) realloc(pointer,newsize);
#endif
  if (!tmp)
  {
    err(memer);
    oldsize = 0; /* to keep compiler happy */
  }
  return tmp;
}
ENDEXTERN

GEN
allocatemem(unsigned long newsize)
{
#if __MWERKS__
  newsize = newsize; /* to avoid "unused variable" msg */
  err(talker, "Not implemented in this version, sorry");
#else
  allocatemoremem(newsize);
  longjmp(environnement,errpile);
#endif
  return NULL; /* not reached */
}

/* In case dynamic linking would lead to several xmallocs defined. */
#ifdef MALLOC_PROCS
#  undef xmalloc
#  undef xrealloc
#  undef free
   Malloc_procs xmalloc_procs = { xmalloc, xrealloc, free };
#endif

/*******************************************************************/
/*                                                                 */
/*                               TIMER                             */
/*                                                                 */
/*******************************************************************/
#define MAX_TIMER 2

#ifdef macintosh
# include <Events.h>

  static long
  timer_proto(int i)
  {
    static long oldticks[MAX_TIMER];
    long ticks = TickCount(), delay = ticks - oldticks[i];

    oldticks[i] = ticks;
    return 50 * delay / 3;
  }

#elif USE_TIMES
# define _INCLUDE_POSIX_SOURCE
# include <sys/times.h>
# include <sys/time.h>
  static long
  timer_proto(int i)
  {
    static clock_t oldticks[MAX_TIMER];
    clock_t delay;
    struct tms t;

    times(&t);
    delay = (t.tms_utime - oldticks[i]) * (1000 / CLK_TCK);
    oldticks[i] = t.tms_utime;
    return (long) delay;
  }

#elif USE_GETRUSAGE
# include <sys/time.h>
# include <sys/resource.h>

  static long
  timer_proto(int i)
  {
    static long oldmusec[MAX_TIMER],oldsec[MAX_TIMER];
    long delay;
    struct rusage r;
    struct timeval t;

    getrusage(0,&r); t=r.ru_utime;
    delay = 1000 * (t.tv_sec - oldsec[i]) + (t.tv_usec - oldmusec[i]) / 1000;
    oldmusec[i] = t.tv_usec; oldsec[i] = t.tv_sec;
    return delay;
  }

#elif USE_FTIME
# include <sys/timeb.h>

  static long
  timer_proto(int i)
  {
    static long oldmsec[MAX_TIMER],oldsec[MAX_TIMER];
    long delay;
    struct timeb t;

    ftime(&t);
    delay = 1000 * (t.time - oldsec[i]) + (t.millitm - oldmsec[i]);
    oldmsec[i] = t.millitm; oldsec[i] = t.time;
    return delay;
  }

#else
# include <time.h>
# ifndef CLOCKS_PER_SEC
#   define CLOCKS_PER_SEC 1000000 /* probably false on YOUR system */
# endif
  static long
  timer_proto(int i)
  {
    static clock_t oldclocks[MAX_TIMER];
    clock_t totalclocks = clock();

    unsigned long delay = (totalclocks-oldclocks[i]) * 1000. / CLOCKS_PER_SEC;
    oldclocks[i] = totalclocks;
    return delay;
  }
#endif

long timer() { return timer_proto(0); }
long timer2() { return timer_proto(1); }
